# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.db import transaction
from django.utils.translation import ugettext_lazy as _, ugettext

from cms.api import add_plugin
from cms.utils import permissions
from cms.utils.conf import get_cms_setting
from cms.wizards.wizard_pool import wizard_pool
from cms.wizards.wizard_base import Wizard
from cms.wizards.forms import BaseFormMixin

from djangocms_text_ckeditor.widgets import TextEditorWidget
from djangocms_text_ckeditor.html import clean_html
from parler.forms import TranslatableModelForm
from reversion.revisions import revision_context_manager

from .cms_appconfig import FaqConfig
from .models import Category, Question
from .utils import is_valid_namespace


class ConfigCheckMixin(object):

    def user_has_add_permission(self, user, **kwargs):
        """
        Return True if the current user has permission to add a Category or
        Question (depending on value of `perm_string` class variable.
        :param user: The current user
        :param kwargs: Ignored here
        :return: True if user has add permission, else False
        """
        # No one can create an Article, if there is no app_config yet.
        configs = FaqConfig.objects.all()
        if not configs:
            return False
        if not any([is_valid_namespace(config.namespace)
                    for config in configs]):
            return False

        # Ensure user has permission to create articles.
        if user.is_superuser or user.has_perm(self.perm_string):
            return True

        # By default, no permission.
        return False


class FaqCategoryWizard(ConfigCheckMixin, Wizard):
    perm_string = "aldryn_faq.add_category"


class FaqQuestionWizard(ConfigCheckMixin, Wizard):
    perm_string = "aldryn_faq.add_question"

    def user_has_add_permission(self, user, **kwargs):
        """
        Return True if the current user has permission to add a Question and
        there is at least one category.
        :param user: The current user
        :param kwargs: Ignored here
        :return: True if user has add permission, else False
        """
        base_perm = super(FaqQuestionWizard, self).user_has_add_permission(
            user, **kwargs)
        return base_perm and Category.objects.exists()


class CreateFaqCategoryForm(BaseFormMixin, TranslatableModelForm):
    """
    The ModelForm for the FAQ Category wizard. Note that Category has few
    translated fields that we need to access, so, we use TranslatableModelForm
    """

    class Meta:
        model = Category
        fields = ['name', 'slug', 'appconfig']
        # The natural widget for app_config is meant for normal Admin views and
        # contains JS to refresh the page on change. This is not wanted here.
        widgets = {'appconfig': forms.Select()}

    def __init__(self, **kwargs):
        super(CreateFaqCategoryForm, self).__init__(**kwargs)
        # TODO: FIX this nasty hack to dissallow empty app_config
        if 'appconfig' in self.fields:
            self.fields['appconfig'].required = True
        # If there's only 1 app_config, don't bother show the
        # app_config choice field, we'll choose the option for the user.
        app_configs = FaqConfig.objects.all()
        # check if app config is apphooked
        app_configs = [app_config
                       for app_config in app_configs
                       if is_valid_namespace(app_config.namespace)]
        if len(app_configs) == 1:
            self.fields['appconfig'].widget = forms.HiddenInput()
            self.fields['appconfig'].initial = app_configs[0].pk

    def save(self, commit=True):
        """
        Ensure we create a revision for reversion.
        """
        category = super(CreateFaqCategoryForm, self).save(commit=False)

        # Ensure we make an initial revision
        with transaction.atomic():
            with revision_context_manager.create_revision():
                category.save()
                if self.user:
                    revision_context_manager.set_user(self.user)
                revision_context_manager.set_comment(
                    ugettext("Initial version."))

        return category


class CreateFaqQuestionForm(BaseFormMixin, TranslatableModelForm):
    """
    The ModelForm for the FAQ Question wizard. Note that Category has few
    translated fields that we need to access, so, we use TranslatableModelForm
    """

    answer = forms.CharField(
        label="Answer", required=False, widget=TextEditorWidget,
        help_text=_("Optional. If provided, will be added to the main body of "
                    "the Question answer.")
    )

    class Meta:
        model = Question
        fields = ['title', 'category', 'is_top', 'answer_text',
                  'answer']

    def __init__(self, **kwargs):
        super(CreateFaqQuestionForm, self).__init__(**kwargs)

        # If there's only 1 category, don't bother show the empty label (choice)
        if Category.objects.count() == 1:
            self.fields['category'].empty_label = None

    def save(self, commit=True):
        question = super(CreateFaqQuestionForm, self).save(commit=False)

        # If 'content' field has value, create a TextPlugin with same and add
        # it to the PlaceholderField
        answer = clean_html(self.cleaned_data.get('answer', ''), False)

        try:
            # CMS >= 3.3.x
            content_plugin = get_cms_setting('PAGE_WIZARD_CONTENT_PLUGIN')
        except KeyError:
            # CMS <= 3.2.x
            content_plugin = get_cms_setting('WIZARD_CONTENT_PLUGIN')

        try:
            # CMS >= 3.3.x
            content_field = get_cms_setting('PAGE_WIZARD_CONTENT_PLUGIN_BODY')
        except KeyError:
            # CMS <= 3.2.x
            content_field = get_cms_setting('WIZARD_CONTENT_PLUGIN_BODY')

        if answer and permissions.has_plugin_permission(
                self.user, content_plugin, 'add'):

            # If the question has not been saved, then there will be no
            # Placeholder set-up for this question yet, so, ensure we have saved
            # first.
            if not question.pk:
                question.save()

            if question and question.answer:
                plugin_kwarg = {
                    'placeholder': question.answer,
                    'plugin_type': content_plugin,
                    'language': self.language_code,
                    content_field: answer,
                }
                add_plugin(**plugin_kwarg)

        # Ensure we make an initial revision
        with transaction.atomic():
            with revision_context_manager.create_revision():
                question.save()
                if self.user:
                    revision_context_manager.set_user(self.user)
                revision_context_manager.set_comment(
                    ugettext("Initial version."))

        return question

faq_category_wizard = FaqCategoryWizard(
    title=_(u"New FAQ category"),
    weight=400,
    form=CreateFaqCategoryForm,
    description=_(u"Create a new FAQ category.")
)

wizard_pool.register(faq_category_wizard)

faq_category_wizard = FaqQuestionWizard(
    title=_(u"New FAQ question"),
    weight=450,
    form=CreateFaqQuestionForm,
    description=_(u"Create a new FAQ question.")
)

wizard_pool.register(faq_category_wizard)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__version__ = '1.2.1'

# TODO: Check if these are still required
request_faq_category_identifier = 'aldryn_faq_current_category'
request_faq_question_identifier = 'aldryn_faq_current_question'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0006_auto_20150903_1336'),
    ]

    operations = [
        migrations.AddField(
            model_name='faqconfig',
            name='non_permalink_handling',
            field=models.SmallIntegerField(default=302, help_text='How to handle non-permalink urls?', verbose_name='non-permalink handling', choices=[(200, 'Allow'), (302, 'Redirect to permalink (default)'), (301, 'Permanent redirect to permalink'), (404, 'Return 404: Not Found')]),
        ),
        migrations.AddField(
            model_name='faqconfig',
            name='permalink_type',
            field=models.CharField(default='Ss', help_text='Choose the style of urls to use from the examples. (Note, all types are relative to apphook)', max_length=2, verbose_name='permalink type', choices=[('Pp', '1/2/'), ('Ps', '1/question-slug/'), ('Sp', 'category-slug/2/'), ('Ss', 'category-slug/question-slug/'), ('Bp', '1-category-slug/2/'), ('Bs', '1-category-slug/question-slug/')]),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.template import RequestContext

from aldryn_search.utils import get_index_base, strip_tags
from parler.utils.context import switch_language
from .models import Question, Category


class QuestionIndex(get_index_base()):

    haystack_use_for_indexing = getattr(
        settings, "ALDRYN_FAQ_QUESTION_SEARCH", True)

    index_title = True

    def get_title(self, obj):
        with switch_language(obj):
            return obj.safe_translation_getter('title')

    def get_index_queryset(self, language):
        questions = self.get_model().objects.language(language)
        return questions.active_translations(language)

    def get_model(self):
        return Question

    def get_search_data(self, obj, language, request):
        with switch_language(obj, language):
            context = RequestContext(request)
            text_bits = [
                strip_tags(obj.safe_translation_getter('title') or ''),
                strip_tags(obj.safe_translation_getter('answer_text') or '')
            ]
            plugins = obj.answer.cmsplugin_set.filter(language=language)
            for base_plugin in plugins:
                instance, plugin_type = base_plugin.get_plugin_instance()
                if instance is not None:
                    plugin_content = strip_tags(
                        instance.render_plugin(context=context)
                    )
                    text_bits.append(plugin_content)

            return ' '.join(text_bits)


class CategoryIndex(get_index_base()):

    haystack_use_for_indexing = getattr(
        settings, "ALDRYN_FAQ_CATEGORY_SEARCH", True)

    index_title = True

    def get_title(self, obj):
        with switch_language(obj):
            return obj.safe_translation_getter('name')

    def get_index_queryset(self, language):
        categories = self.get_model().objects.language(language)
        return categories.active_translations(language)

    def get_model(self):
        return Category

    def get_search_data(self, obj, language, request):
        with switch_language(obj):
            return strip_tags(obj.safe_translation_getter('name'))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

from aldryn_faq.utils import rename_tables_new_to_old


class Migration(SchemaMigration):

    def forwards(self, orm):
        rename_tables_new_to_old(db)

        # Adding field 'Question.number_of_visits'
        db.add_column(u'aldryn_faq_question', 'number_of_visits',
                      self.gf('django.db.models.fields.PositiveIntegerField')(default=0),
                      keep_default=False)


    def backwards(self, orm):
        rename_tables_new_to_old(db)

        # Deleting field 'Question.number_of_visits'
        db.delete_column(u'aldryn_faq_question', 'number_of_visits')


    models = {
        u'aldryn_faq.category': {
            'Meta': {'object_name': 'Category'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aldryn_faq.categorylistplugin': {
            'Meta': {'object_name': 'CategoryListPlugin', '_ormbases': ['cms.CMSPlugin']},
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'})
        },
        u'aldryn_faq.categorytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'CategoryTranslation', 'db_table': "u'aldryn_faq_category_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['aldryn_faq.Category']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'})
        },
        u'aldryn_faq.latestquestionsplugin': {
            'Meta': {'object_name': 'LatestQuestionsPlugin', '_ormbases': ['cms.CMSPlugin']},
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'questions': ('django.db.models.fields.IntegerField', [], {'default': '5'})
        },
        u'aldryn_faq.question': {
            'Meta': {'ordering': "['order']", 'object_name': 'Question'},
            'answer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'faq_questions'", 'null': 'True', 'to': "orm['cms.Placeholder']"}),
            'category': ('adminsortable.fields.SortableForeignKey', [], {'related_name': "'questions'", 'to': u"orm['aldryn_faq.Category']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_top': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'number_of_visits': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'})
        },
        u'aldryn_faq.questionlistplugin': {
            'Meta': {'object_name': 'QuestionListPlugin', '_ormbases': ['cms.CMSPlugin']},
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'questions': ('sortedm2m.fields.SortedManyToManyField', [], {'to': u"orm['aldryn_faq.Question']", 'symmetrical': 'False'})
        },
        u'aldryn_faq.questiontranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'QuestionTranslation', 'db_table': "u'aldryn_faq_question_translation'"},
            'answer_text': ('djangocms_text_ckeditor.fields.HTMLField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['aldryn_faq.Question']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'aldryn_faq.selectedcategory': {
            'Meta': {'ordering': "['position']", 'object_name': 'SelectedCategory'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['aldryn_faq.Category']"}),
            'cms_plugin': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'selected_categories'", 'to': u"orm['aldryn_faq.CategoryListPlugin']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'aldryn_faq.topquestionsplugin': {
            'Meta': {'object_name': 'TopQuestionsPlugin', '_ormbases': ['cms.CMSPlugin']},
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'questions': ('django.db.models.fields.IntegerField', [], {'default': '5'})
        },
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_faq']


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from cms.utils.conf import get_cms_setting

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(
        r'^media/(?P<path>.*)$', 'django.views.static.serve', {
            'document_root': settings.MEDIA_ROOT, 'show_indexes': True
        }
    ),
    url(
        r'^media/cms/(?P<path>.*)$',
        'django.views.static.serve',
        {
            'document_root': get_cms_setting('MEDIA_ROOT'),
            'show_indexes': True
        }
    ),
    url(
        r'^jsi18n/(?P<packages>\S+?)/$',
        'django.views.i18n.javascript_catalog'
    ),
)

url_i18n_patterns = i18n_patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^faq/', include('aldryn_faq.urls', namespace='aldryn_faq')),
    url(r'^', include('cms.urls')),
)

urlpatterns += staticfiles_urlpatterns()

urlpatterns += url_i18n_patterns



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import random
import string

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import RequestFactory, TransactionTestCase
from django.utils.translation import override

from cms import api
from cms.models import Title
from cms.utils import get_cms_setting
from cms.utils.i18n import get_language_list
from djangocms_helper.utils import create_user

from ..models import Category, Question, FaqConfig

User = get_user_model()


class TestUtilityMixin(object):
    """Just adds some common test utilities to the testing class."""
    def rand_str(self, prefix='', length=16, chars=string.ascii_letters):
        return prefix + ''.join(random.choice(chars) for _ in range(length))

    def assertEqualItems(self, a, b):
        try:
            # In Python3, this method has been renamed (poorly)
            assertCountEqual = self.assertCountEqual
        except AttributeError:
            # In 2.6, assertItemsEqual() doesn't sort first
            def assertCountEqual(a, b):
                return self.assertItemsEqual(sorted(a), sorted(b))

        return assertCountEqual(a, b)


class AldrynFaqTestMixin(TestUtilityMixin, object):
    """Sets up basic Category and Question objects for testing."""
    data = {
        "category1": {
            "en": {"name": "Example1", "slug": "example", },
            "de": {"name": "Beispiel1", "slug": "beispiel", }
        },
        "category2": {
            # This should *not* have a EN translation
            "de": {"name": "Beispiel2", "slug": "beispiel2", }
        },
        "question1": {
            "en": {"title": "Test Question", "answer_text": "Test Answer",
                   "slug": "test-question", },
            "de": {"title": "Testfrage", "answer_text": "Test Antwort",
                   "slug": "Testfrage", },
        },
        "question2": {
            # This should *not* have a EN translation
            "de": {"title": "Testfrage2", "answer_text": "Test Antwort2",
                   "slug": "testfrage2", },
        },
    }

    enabled_parler_fallback_settings = {
        'PARLER_LANGUAGES': {
            1: (
                {'code': 'de', 'fallbacks': ['en', ]},
                {'code': 'en', 'fallbacks': ['de', ]},
                {'code': 'fr', 'fallbacks': ['en', ]},
            ),
            'default': {
                'hide_untranslated': False,
            }
        }
    }

    disabled_parler_fallback_settings = {
        'PARLER_LANGUAGES': {
            1: (
                {'code': 'de'},
                {'code': 'en'},
                {'code': 'fr'},
            ),
            'default': {
                'hide_untranslated': False,
            }
        }
    }

    settings_en = {
        'LANGUAGES': (
            ('en', 'English'),
        ),
        'LANGUAGE': 'en',
        'PARLER_LANGUAGES': {
            1: (
                {'code': 'en'},
            ),
            'default': {
                'hide_untranslated': False,
            },
        },
        'CMS_LANGUAGES': {
            1: (
                {'code': 'en', 'name': 'English'},
            ),
            'default': {
                'hide_untranslated': False,
            },
        },
    }

    @staticmethod
    def reload(obj, language=None):
        """Simple convenience method for re-fetching an object from the ORM,
        optionally "as" a specified language."""
        if language:
            with override(language):
                new_obj = obj.__class__.objects.get(id=obj.id)
        else:
            new_obj = obj.__class__.objects.get(id=obj.id)
        return new_obj

    def mktranslation(self, obj, lang, **kwargs):
        """Simple method of adding a translation to an existing object."""
        obj.set_current_language(lang)
        for k, v in kwargs.items():
            setattr(obj, k, v)
        obj.save()

    def setUp(self):
        """Setup a prebuilt and translated Question with Category
        for testing."""
        super(AldrynFaqTestMixin, self).setUp()
        with override("en"):
            self.category1 = Category(**self.data["category1"]["en"])
            self.category1.appconfig = self.app_config
            self.category1.save()
            self.question1 = Question(**self.data["question1"]["en"])
            self.question1.category = self.category1
            self.question1.save()

        # Make a DE translation of the category
        self.mktranslation(
            self.category1,
            "de",
            **self.data["category1"]["de"]
        )

        # Make a DE translation of the question
        self.mktranslation(
            self.question1,
            "de",
            **self.data["question1"]["de"]
        )

        with override("de"):
            # Make a DE-only Category
            self.category2 = Category(**self.data["category2"]["de"])
            self.category2.appconfig = self.app_config
            self.category2.save()

            # Make a DE-only Question
            self.question2 = Question(**self.data["question2"]["de"])
            self.question2.category = self.category2
            self.question2.save()


class CMSRequestBasedTest(TestUtilityMixin, TransactionTestCase):
    """Sets-up User(s) and CMS Pages for testing."""
    languages = get_language_list()

    @classmethod
    def setUpClass(cls):
        cls.request_factory = RequestFactory()
        if not User.objects.filter(username='normal').count():
            cls.user = create_user('normal', 'normal@admin.com', 'normal')
        cls.site1 = Site.objects.get(pk=1)

    @classmethod
    def tearDownClass(cls):
        User.objects.all().delete()

    def setUp(self):
        super(CMSRequestBasedTest, self).setUp()
        self.template = get_cms_setting('TEMPLATES')[0][0]
        self.language = settings.LANGUAGES[0][0]
        self.root_page = api.create_page(
            'root page',
            self.template,
            self.language,
            published=True
        )
        self.app_config = FaqConfig.objects.create(namespace='aldryn_faq',
                                                   permalink_type='Bp',
                                                   non_permalink_handling=301)
        self.page = api.create_page(
            'faq',
            self.template,
            self.language,
            published=True,
            parent=self.root_page,
            apphook='FaqApp',
            apphook_namespace=self.app_config.namespace
        )
        self.placeholder = self.page.placeholders.all()[0]

        for page in [self.root_page, self.page]:
            for language, _ in settings.LANGUAGES[1:]:
                api.create_title(language, page.get_slug(), page)
                page.publish(language)

    def get_or_create_page(self, base_title=None, languages=None):
        """Creates a page with a given title, or, if it already exists, just
        retrieves and returns it."""
        from cms.api import create_page, create_title
        if not base_title:
            # No title? Create one.
            base_title = self.rand_str(prefix="page", length=8)
        if not languages:
            # If no langs supplied, use'em all
            languages = self.languages
        # If there is already a page with this title, just return it.
        try:
            page_title = Title.objects.get(title=base_title)
            return page_title.page.get_draft_object()
        except:
            pass

        # No? Okay, create one.
        page = create_page(base_title, 'fullwidth.html', language=languages[0])
        # If there are multiple languages, create the translations
        if len(languages) > 1:
            for lang in languages[1:]:
                title_lang = "{0}-{1}".format(base_title, lang)
                create_title(language=lang, title=title_lang, page=page)
                page.publish(lang)
        return page.get_draft_object()

    def get_page_request(
            self, page, user, path=None, edit=False, lang_code='en'):
        from cms.middleware.toolbar import ToolbarMiddleware
        path = path or page and page.get_absolute_url()
        if edit:
            path += '?edit'
        request = RequestFactory().get(path)
        request.session = {}
        request.user = user
        request.LANGUAGE_CODE = lang_code
        if edit:
            request.GET = {'edit': None}
        else:
            request.GET = {'edit_off': None}
        request.current_page = page
        mid = ToolbarMiddleware()
        mid.process_request(request)
        return request


class AldrynFaqTest(AldrynFaqTestMixin, CMSRequestBasedTest):
    pass



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from django.core.urlresolvers import reverse, NoReverseMatch

default_cms_plugin_table_mapping = (
    # (old_name, new_name),
    ('cmsplugin_categorylistplugin', 'aldryn_faq_categorylistplugin'),
    ('cmsplugin_latestquestionsplugin', 'aldryn_faq_latestquestionsplugin'),
    ('cmsplugin_mostreadquestionsplugin',
     'aldryn_faq_mostreadquestionsplugin'),
    ('cmsplugin_questionlistplugin', 'aldryn_faq_questionlistplugin'),
    ('cmsplugin_questionlistplugin_questions',
     'aldryn_faq_questionlistplugin_questions'),
    ('cmsplugin_topquestionsplugin', 'aldryn_faq_topquestionsplugin'),
)


def rename_tables(db, table_mapping=None, reverse=False):
    """
    renames tables from source to destination name, if the source exists
    and the destination does not exist yet.

    taken from cmsplugin-filer:cmsplugin_filer_utils.migration
    (thanks to @stefanfoulis)
    """
    from django.db import connection

    if not table_mapping:
        table_mapping = default_cms_plugin_table_mapping

    if reverse:
        table_mapping = [(dst, src) for src, dst in table_mapping]
    table_names = connection.introspection.table_names()
    for source, destination in table_mapping:
        if source in table_names and destination in table_names:
            print("    WARNING: not renaming {0} to {1}, because both "
                  "tables already exist.".format(source, destination))
        elif source in table_names and destination not in table_names:
            print("     - renaming {0} to {1}".format(source, destination))
            db.rename_table(source, destination)


def rename_tables_old_to_new(db, table_mapping=None):
    return rename_tables(db, table_mapping, reverse=False)


def rename_tables_new_to_old(db, table_mapping=None):
    return rename_tables(db, table_mapping, reverse=True)


def is_valid_namespace(namespace):
    """
    Check if provided namespace has an app-hooked page.
    Returns True or False.
    """
    try:
        reverse('{0}:faq-category-list'.format(namespace))
    except (NoReverseMatch, AttributeError):
        return False
    return True


def is_valid_app_config(app_config):
    """
    Checks if provided app_config is valid. Tries to get namespace.
    """
    return getattr(app_config, 'namespace', None)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

try:
    from imp import reload
except ImportError:
    from importlib import reload

from django.core.urlresolvers import resolve, reverse
from django.http import Http404
from django.utils.translation import override

import parler.appsettings

from ..models import Category
from ..views import FaqByCategoryView, FaqAnswerView

from .test_base import AldrynFaqTest


class TestFaqByCategoryView(AldrynFaqTest):

    def test_view_context(self):
        """Tests that the FaqByCategoryView produces the correct context."""
        category_1 = self.reload(self.category1, "en")
        category_1_url = category_1.get_absolute_url()

        question_1 = self.reload(self.question1, "en")

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=category_1_url,
        )

        url_kwargs = resolve(category_1_url).kwargs

        try:
            response = FaqByCategoryView.as_view()(request, **url_kwargs)
        except Http404:
            self.fail('Could not find category')

        self.assertEqualItems(
            response.context_data['object_list'],
            [question_1, ],
        )

    def test_view_context_fallback(self):
        """
        Tests that the FaqByCategoryView produces the correct context
        when requesting a category in an untranslated language.
        """
        category_2 = self.reload(self.category2, "en")
        category_2_url = category_2.get_absolute_url()

        question_2 = self.reload(self.question2, "en")

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=category_2_url,
        )

        url_kwargs = resolve(category_2_url).kwargs

        with self.settings(**self.enabled_parler_fallback_settings):
            reload(parler.appsettings)

            try:
                response = FaqByCategoryView.as_view()(request, **url_kwargs)
            except Http404:
                self.fail('Could not find category')

            self.assertEqualItems(
                response.context_data['object_list'],
                [question_2, ],
            )

    def test_view_old_format_redirect(self):
        """
        Tests that the FaqByCategoryView redirects user
        when accessed with old category url format
        """
        category_1 = self.reload(self.category1, "en")
        category_1_url_new = category_1.get_absolute_url()

        kwargs = {"category_slug": category_1.slug}

        with override('en'):
            category_1_url_name = '{ns}:faq-category'.format(
                ns=self.app_config.namespace
            )
            category_1_url_old = reverse(category_1_url_name, kwargs=kwargs)

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=category_1_url_old,
        )

        response = FaqByCategoryView.as_view()(request, **kwargs)

        self.assertEquals(response.status_code, 301)
        self.assertEquals(response.url, category_1_url_new)

    def test_list_view(self):
        """Test category list view to contain a proper set of categories"""
        def _do_test_list_view(language_code):
            with override(language_code):
                categories = Category.objects.language(
                    language_code).active_translations(language_code).filter(
                        appconfig=self.app_config)
                url = reverse('{ns}:faq-category-list'.format(
                    ns=self.app_config.namespace))
                response = self.client.get(url, follow=True)
                for category in categories:
                    self.assertContains(response, category.name)

        for language_code in ('en', 'de'):
            _do_test_list_view(language_code)

        with self.settings(**self.settings_en):
            reload(parler.appsettings)
            _do_test_list_view('en')


class TestFaqAnswerView(AldrynFaqTest):

    def test_view_context(self):
        """Tests that the FaqByCategoryView produces the correct context."""
        question_1 = self.reload(self.question1, "en")
        question_1_url = question_1.get_absolute_url("en")

        url_kwargs = resolve(question_1_url).kwargs

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=question_1_url,
        )

        response = FaqAnswerView.as_view()(request, **url_kwargs)

        self.assertEqual(
            response.context_data['object'],
            question_1,
        )

    def test_view_context_fallback(self):
        """
        Tests that the FaqByCategoryView produces the correct context
        when requesting a category in an untranslated language.
        """
        question_2 = self.reload(self.question1, "en")
        question_2_url = question_2.get_absolute_url("en")

        url_kwargs = resolve(question_2_url).kwargs

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=question_2_url,
        )

        with self.settings(**self.enabled_parler_fallback_settings):
            reload(parler.appsettings)
            response = FaqAnswerView.as_view()(request, **url_kwargs)

        self.assertEqual(
            response.context_data['object'],
            question_2,
        )

    def test_view_old_format_redirect(self):
        """
        Tests that the TestFaqAnswerView redirects user
        when accessed with old category url format
        """
        category_1 = self.reload(self.category1, "en")
        question_1 = self.reload(self.question1, "en")
        question_1_url_new = question_1.get_absolute_url()

        kwargs = {
            "category_slug": category_1.slug,
            "pk": question_1.pk
        }

        with override('en'):
            url_name = '{ns}:faq-answer'.format(ns=self.app_config.namespace)
            question_1_url_old = reverse(url_name, kwargs=kwargs)

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=question_1_url_old,
        )

        response = FaqAnswerView.as_view()(request, **kwargs)

        self.assertEquals(response.status_code, 301)
        self.assertEquals(response.url, question_1_url_new)

    def test_answer_match_category(self):
        """
        Tests that the question id given in url
        belongs to the given category, if not then 404 is raised.
        """
        category_1 = self.reload(self.category1, "de")
        question_2 = self.reload(self.question2, "de")

        kwargs = {
            "category_pk": category_1.pk,
            "category_slug": category_1.slug,
            "pk": question_2.pk
        }

        with override('de'):
            url_name = '{ns}:faq-answer'.format(ns=self.app_config.namespace)
            question_2_invalid_url = reverse(url_name, kwargs=kwargs)

        request = self.get_page_request(
            page=self.page,
            user=self.user,
            path=question_2_invalid_url,
        )

        with self.assertRaises(Http404):
            FaqAnswerView.as_view()(request, **kwargs)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import force_text
from django.utils.translation import override

from aldryn_faq.models import Category, Question, get_slug_in_language

from .test_base import AldrynFaqTest


class TestCategory(AldrynFaqTest):

    def test_unicode(self):
        with override('en'):
            category1 = self.reload(self.category1)
            self.assertEqual(
                force_text(category1), self.data["category1"]["en"]["name"])

        with override('de'):
            category1 = self.reload(self.category1)
            self.assertEqual(
                force_text(category1), self.data["category1"]["de"]["name"])

    def test_get_slug_in_language(self):
        self.assertIsNone(get_slug_in_language(None, 'en'), None)
        self.assertIsNone(get_slug_in_language(object, 'en'), None)
        self.assertEqual(
            get_slug_in_language(self.category1, 'en'),
            self.data["category1"]["en"]["slug"]
        )
        self.assertEqual(
            get_slug_in_language(self.category1, 'de'),
            self.data["category1"]["de"]["slug"]
        )

    def test_model_type_id(self):
        ct = ContentType.objects.get(app_label='aldryn_faq', model='category')
        self.assertEqual(
            self.category1.model_type_id(),
            ct.pk
        )

    def test_get_absolute_url_pk_pk(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Pp"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_pk}/{pk}/".format(
                category_pk=category_1.pk,
                pk=question_1.pk)
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_pk}/{pk}/".format(
                category_pk=category_1.pk,
                pk=question_1.pk)
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_get_absolute_url_pk_slug(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Ps"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_pk}/{slug}/".format(
                category_pk=category_1.pk,
                slug=get_slug_in_language(question_1, "en"))
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_pk}/{slug}/".format(
                category_pk=category_1.pk,
                slug=get_slug_in_language(question_1, "de"))
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_get_absolute_url_slug_pk(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Sp"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_slug}/{pk}/".format(
                category_slug=get_slug_in_language(category_1, "en"),
                pk=question_1.pk
            )
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_slug}/{pk}/".format(
                category_slug=get_slug_in_language(category_1, "de"),
                pk=question_1.pk
            )
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_get_absolute_url_slug_slug(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Ss"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_slug}/{slug}/".format(
                category_slug=get_slug_in_language(category_1, "en"),
                slug=get_slug_in_language(question_1, "en")
            )
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_slug}/{slug}/".format(
                category_slug=get_slug_in_language(category_1, "de"),
                slug=get_slug_in_language(question_1, "de")
            )
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_get_absolute_url_both_pk(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Bp"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_pk}-{category_slug}/{pk}/".format(
                category_slug=get_slug_in_language(category_1, "en"),
                category_pk=category_1.pk,
                pk=question_1.pk
            )
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_pk}-{category_slug}/{pk}/".format(
                category_slug=get_slug_in_language(category_1, "de"),
                category_pk=category_1.pk,
                pk=question_1.pk
            )
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_get_absolute_url_both_slug(self):
        question_1 = self.question1
        category_1 = self.category1

        old = self.app_config.permalink_type
        self.app_config.permalink_type = "Bs"
        self.app_config.save()

        self.assertEqual(
            question_1.get_absolute_url("en"),
            "/en/faq/{category_pk}-{category_slug}/{slug}/".format(
                category_slug=get_slug_in_language(category_1, "en"),
                category_pk=category_1.pk,
                slug=get_slug_in_language(question_1, "en"))
        )
        self.assertEqual(
            question_1.get_absolute_url("de"),
            "/de/faq/{category_pk}-{category_slug}/{slug}/".format(
                category_slug=get_slug_in_language(category_1, "de"),
                category_pk=category_1.pk,
                slug=get_slug_in_language(question_1, "de"))
        )

        self.app_config.permalink_type = old
        self.app_config.save()

    def test_manager_get_categories(self):
        # Test case when no language is passed. Apparently, this returns only
        # those objects that have the default language translations.
        # TODO: Verify that this is the intended behavior.
        # self.assertEqualItems(
        #     [c.pk for c in Category.objects.get_categories()],
        #     [self.category1.pk, ]
        # )

        # Test case of requesting objects of only a single language. In our
        # setup, we have 1 category in EN and 2 in DE.
        categories = Category.objects.get_categories('en')
        self.assertEqualItems(
            [category.pk for category in categories],
            [self.category1.pk, ]
        )

        # There's actually two categories in DE
        categories = Category.objects.get_categories('de')
        cids = set([category.pk for category in categories])
        self.assertEqual(
            cids,
            set([self.category1.pk, self.category2.pk])
        )


class TestQuestion(AldrynFaqTest):

    def test_unicode(self):
        with override('en'):
            question1 = self.reload(self.question1)
            self.assertEqual(
                force_text(question1),
                self.data["question1"]["en"]["title"]
            )
        with override('de'):
            question1 = self.reload(self.question1)
            self.assertEqual(
                force_text(question1),
                self.data["question1"]["de"]["title"]
            )

    def test_model_type_id(self):
        ct = ContentType.objects.get(app_label='aldryn_faq', model='question')
        self.assertEqual(
            self.question1.model_type_id(),
            ct.pk
        )

    def test_get_absolue_url(self):
        category_pk = self.question1.category_id
        question_1_pk = self.question1.pk
        question_1_url_en = "/en/faq/{cat_pk}-example/{pk}/".format(
            cat_pk=category_pk,
            pk=question_1_pk
        )
        question_1_url_de = "/de/faq/{cat_pk}-beispiel/{pk}/".format(
            cat_pk=category_pk,
            pk=question_1_pk
        )

        self.assertEqual(
            self.question1.get_absolute_url("en"),
            question_1_url_en
        )
        self.assertEqual(
            self.question1.get_absolute_url("de"),
            question_1_url_de
        )

    def test_manager_filter_by_language(self):
        questions = Question.objects.filter_by_language('en')
        self.assertEqualItems(
            [q.pk for q in questions],
            [self.question1.pk]
        )

        questions = Question.objects.filter_by_language('de')
        self.assertEqualItems(
            [q.pk for q in questions],
            [self.question1.pk, self.question2.pk]
        )

    def test_manager_filter_by_current_language(self):
        with override("en"):
            questions = Question.objects.filter_by_current_language()
            self.assertEqualItems(
                [q.pk for q in questions],
                [self.question1.pk]
            )

        with override("de"):
            questions = Question.objects.filter_by_current_language()
            self.assertEqualItems(
                [q.pk for q in questions],
                [self.question1.pk, self.question2.pk]
            )


class TestFAQTranslations(AldrynFaqTest):

    def test_fetch_faq_translations(self):
        """Test we can fetch arbitrary translations of the question and
        its category."""
        # Can we target the EN values?
        with override("en"):
            question1 = self.reload(self.question1)
            category1 = self.reload(self.question1.category)
            self.assertEqual(
                question1.title,
                self.data["question1"]["en"]["title"]
            )
            self.assertEqual(
                question1.answer_text,
                self.data["question1"]["en"]["answer_text"]
            )
            self.assertEqual(
                category1.name,
                self.data["category1"]["en"]["name"]
            )
            self.assertEqual(
                category1.slug,
                self.data["category1"]["en"]["slug"]
            )

        # And the DE values?
        with override("de"):
            question1 = self.reload(self.question1)
            category1 = self.reload(self.question1.category)
            self.assertEqual(
                question1.title,
                self.data["question1"]["de"]["title"]
            )
            self.assertEqual(
                question1.answer_text,
                self.data["question1"]["de"]["answer_text"]
            )
            self.assertEqual(
                category1.name,
                self.data["category1"]["de"]["name"]
            )
            self.assertEqual(
                category1.slug,
                self.data["category1"]["de"]["slug"]
            )

    def test_fetch_faq_translations_fallbacks(self):
        """Test we can fetch arbitrary translations of the question and
        its category."""

        # Question and Category 1 do not exist in french
        # we expect to fallback to english
        with override("fr"):
            question_1_fr = self.reload(self.question1)
            question_1_fr_translation = question_1_fr.safe_translation_getter

            category_1_fr = self.reload(self.question1.category)
            category_1_fr_translation = category_1_fr.safe_translation_getter

            self.assertEqual(
                question_1_fr_translation('title', any_language=True),
                self.data["question1"]["en"]["title"]
            )
            self.assertEqual(
                question_1_fr_translation('answer_text', any_language=True),
                self.data["question1"]["en"]["answer_text"]
            )
            self.assertEqual(
                category_1_fr_translation('name', any_language=True),
                self.data["category1"]["en"]["name"]
            )
            self.assertEqual(
                category_1_fr_translation('slug', any_language=True),
                self.data["category1"]["en"]["slug"]
            )

        # Question and Category 2 do not exist in english
        # we expect to fallback to german
        with override("en"):
            question_2_en = self.reload(self.question2)
            question_2_en_translation = question_2_en.safe_translation_getter

            category_2_en = self.reload(self.question2.category)
            category_2_en_translation = category_2_en.safe_translation_getter

            self.assertEqual(
                question_2_en_translation('title', any_language=True),
                self.data["question2"]["de"]["title"]
            )
            self.assertEqual(
                question_2_en_translation('answer_text', any_language=True),
                self.data["question2"]["de"]["answer_text"]
            )
            self.assertEqual(
                category_2_en_translation('name', any_language=True),
                self.data["category2"]["de"]["name"]
            )
            self.assertEqual(
                category_2_en_translation('slug', any_language=True),
                self.data["category2"]["de"]["slug"]
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import cms.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0003_auto_20140926_2347'),
        ('aldryn_faq', '0003_preset_order'),
    ]

    operations = [
        migrations.AddField(
            model_name='faqconfig',
            name='placeholder_faq_content',
            field=cms.models.fields.PlaceholderField(related_name='aldryn_faq_content', slotname='faq_content', editable=False, to='cms.Placeholder', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='faqconfig',
            name='placeholder_faq_list_bottom',
            field=cms.models.fields.PlaceholderField(related_name='aldryn_faq_list_bottom', slotname='faq_list_bottom', editable=False, to='cms.Placeholder', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='faqconfig',
            name='placeholder_faq_list_top',
            field=cms.models.fields.PlaceholderField(related_name='aldryn_faq_list_top', slotname='faq_list_top', editable=False, to='cms.Placeholder', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='faqconfig',
            name='placeholder_faq_sidebar_bottom',
            field=cms.models.fields.PlaceholderField(related_name='aldryn_faq_sidebar_bottom', slotname='faq_sidebar_bottom', editable=False, to='cms.Placeholder', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='faqconfig',
            name='placeholder_faq_sidebar_top',
            field=cms.models.fields.PlaceholderField(related_name='aldryn_faq_sidebar_top', slotname='faq_sidebar_top', editable=False, to='cms.Placeholder', null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import aldryn_translation_tools.models
import app_data.fields
import cms.models.fields
import sortedm2m.fields
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0003_auto_20140926_2347'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
            bases=(aldryn_translation_tools.models.TranslationHelperMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CategoryListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='CategoryTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(max_length=15, verbose_name='Language', db_index=True)),
                ('name', models.CharField(max_length=255)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('master', models.ForeignKey(related_name='translations', editable=False, to='aldryn_faq.Category', null=True)),
            ],
            options={
                'managed': True,
                'db_table': 'aldryn_faq_category_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'category Translation',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FaqConfig',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='type')),
                ('namespace', models.CharField(default=None, unique=True, max_length=100, verbose_name='instance namespace')),
                ('app_data', app_data.fields.AppDataField(default=dict, editable=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FaqConfigTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(max_length=15, verbose_name='Language', db_index=True)),
                ('app_title', models.CharField(max_length=234, verbose_name='application title')),
                ('master', models.ForeignKey(related_name='translations', editable=False, to='aldryn_faq.FaqConfig', null=True)),
            ],
            options={
                'managed': True,
                'db_table': 'aldryn_faq_faqconfig_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'faq config Translation',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='LatestQuestionsPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('questions', models.IntegerField(default=5, help_text='The number of questions to be displayed.')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin', models.Model),
        ),
        migrations.CreateModel(
            name='MostReadQuestionsPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('questions', models.IntegerField(default=5, help_text='The number of questions to be displayed.')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin', models.Model),
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_top', models.BooleanField(default=False)),
                ('number_of_visits', models.PositiveIntegerField(default=0, editable=False)),
                ('order', models.PositiveIntegerField(default=1, db_index=True)),
                ('answer', cms.models.fields.PlaceholderField(related_name='faq_questions', slotname='faq_question_answer', editable=False, to='cms.Placeholder', null=True)),
                ('category', models.ForeignKey(related_name='questions', to='aldryn_faq.Category')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'question',
                'verbose_name_plural': 'questions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='QuestionListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('questions', sortedm2m.fields.SortedManyToManyField(help_text=None, to='aldryn_faq.Question')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='QuestionTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(max_length=15, verbose_name='Language', db_index=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('answer_text', djangocms_text_ckeditor.fields.HTMLField(verbose_name='answer')),
                ('master', models.ForeignKey(related_name='translations', editable=False, to='aldryn_faq.Question', null=True)),
            ],
            options={
                'managed': True,
                'db_table': 'aldryn_faq_question_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'question Translation',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SelectedCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('position', models.PositiveIntegerField(null=True, verbose_name='position', blank=True)),
                ('category', models.ForeignKey(verbose_name='category', to='aldryn_faq.Category')),
                ('cms_plugin', models.ForeignKey(related_name='selected_categories', to='aldryn_faq.CategoryListPlugin')),
            ],
            options={
                'ordering': ['position'],
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TopQuestionsPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('questions', models.IntegerField(default=5, help_text='The number of questions to be displayed.')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin', models.Model),
        ),
        migrations.AlterUniqueTogether(
            name='questiontranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='faqconfigtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='categorytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AddField(
            model_name='category',
            name='appconfig',
            field=models.ForeignKey(verbose_name='appconfig', blank=True, to='aldryn_faq.FaqConfig', null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode

from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _, get_language_from_request

from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar
from cms.utils.i18n import force_language
from cms.utils.urlutils import admin_reverse

from aldryn_apphooks_config.utils import get_app_instance
from parler.models import TranslatableModel

from .models import Category, FaqConfig, Question


def get_obj_from_request(model, request,
                         pk_url_kwarg='pk',
                         slug_url_kwarg='slug',
                         slug_field='slug'):
    """
    Given a model and the request, try to extract and return an object
    from an available 'pk' or 'slug', or return None.

    Note that no checking is done that the view's kwargs really are for objects
    matching the provided model (how would it?) so use only where appropriate.
    """
    language = get_language_from_request(request, check_path=True)
    kwargs = request.resolver_match.kwargs
    mgr = model.objects
    if pk_url_kwarg in kwargs:
        return mgr.filter(pk=kwargs[pk_url_kwarg]).first()
    elif slug_url_kwarg in kwargs:
        # If the model is translatable, and the given slug is a translated
        # field, then find it the Parler way.
        filter_kwargs = {slug_field: kwargs[slug_url_kwarg]}
        translated_fields = model._parler_meta.get_translated_fields()
        if (issubclass(model, TranslatableModel) and
                slug_field in translated_fields):
            return mgr.active_translations(language, **filter_kwargs).first()
        else:
            # OK, do it the normal way.
            return mgr.filter(**filter_kwargs).first()
    else:
        return None


def get_admin_url(action, action_args=[], **url_args):
    """
    Convenience method for constructing admin-urls with GET parameters.

    :param action:      The admin url key for use in reverse. E.g.,
                        'aldryn_newsblog_edit_article'
    :param action_args: The url args for the reverse. E.g., [article.pk, ]
    :param url_args:    A dict of key/value pairs for GET parameters. E.g.,
                        {'language': 'en', }.
    :return: The complete admin url
    """
    base_url = admin_reverse(action, args=action_args)
    # Converts [{key: value}, …] => ["key=value", …]
    params = urlencode(url_args)
    if params:
        return "?".join([base_url, params])
    else:
        return base_url


@toolbar_pool.register
class FaqToolbar(CMSToolbar):
    watch_models = (Category, )
    config = None

    def __get_faq_config(self):
        try:
            __, config = get_app_instance(self.request)
            if not isinstance(config, FaqConfig):
                # This is not the app_hook you are looking for.
                return None
        except ImproperlyConfigured:
            # There is no app_hook at all.
            return None

        return config

    def get_category_list_url(self, lang=None):
        """
        Returns url for faq-category-list view with respect to lang or
        current_lang
        """
        with force_language(lang if lang else self.current_lang):
            url = reverse('{0}:faq-category-list'.format(
                self.config.namespace))
        return url

    def get_on_delete_redirect_url(self, obj, lang=None):
        if not self.config:
            self.config = self.__get_faq_config()

        if isinstance(obj, Category):
            url = self.get_category_list_url(lang)
        elif isinstance(obj, Question):
            category = obj.category
            lang = lang if lang else self.current_lang
            if category:
                url = category.get_absolute_url(language=lang)
            else:
                url = self.get_category_list_url(lang)
        return url

    def populate(self):
        self.config = self.__get_faq_config()
        user = getattr(self.request, 'user', None)
        try:
            view_name = self.request.resolver_match.view_name
        except AttributeError:
            view_name = None

        if user and view_name and self.config:
            language = get_language_from_request(self.request, check_path=True)

            category = get_obj_from_request(Category, self.request,
                                            pk_url_kwarg='category_pk',
                                            slug_url_kwarg='category_slug')

            question = get_obj_from_request(Question, self.request)

            menu = self.toolbar.get_or_create_menu('faq-app',
                                                   self.config.get_app_title())

            change_config_perm = user.has_perm('aldryn_faq.change_faqconfig')
            config_perms = [change_config_perm, ]

            add_category_perm = user.has_perm('aldryn_faq.add_category')
            change_category_perm = user.has_perm('aldryn_faq.change_category')
            delete_category_perm = user.has_perm('aldryn_faq.delete_category')
            category_perms = [add_category_perm, change_category_perm,
                              delete_category_perm]

            add_question_perm = user.has_perm('aldryn_faq.add_question')
            change_question_perm = user.has_perm('aldryn_faq.change_question')
            delete_question_perm = user.has_perm('aldryn_faq.delete_question')
            question_perms = [add_question_perm, change_question_perm,
                              delete_question_perm]

            # ------ App Config items -----------------------------------------

            if change_config_perm:
                url_args = {}
                if language:
                    url_args = {'language': language, }
                url = get_admin_url('aldryn_faq_faqconfig_change',
                                    [self.config.pk, ], **url_args)
                menu.add_modal_item(_('Configure addon'), url=url)

            if any(config_perms) and any(category_perms + question_perms):
                menu.add_break()

            # ------ Category items -------------------------------------------

            if change_category_perm:
                url_args = {'appconfig__id__exact': self.config.pk}
                url = get_admin_url('aldryn_faq_category_changelist',
                                    **url_args)
                menu.add_sideframe_item(_('Category list'), url=url)

            if add_category_perm:
                url_args = {'appconfig': self.config.pk, }
                if language:
                    url_args.update({'language': language, })
                url = get_admin_url('aldryn_faq_category_add', **url_args)
                menu.add_modal_item(_('Add new category'), url=url)

            if change_category_perm and category:
                url_args = {}
                if language:
                    url_args = {'language': language, }
                url = get_admin_url('aldryn_faq_category_change',
                                    [category.pk, ], **url_args)
                menu.add_modal_item(_('Edit this category'), url=url,
                                    active=True)

            if delete_category_perm and category:
                redirect_url = self.get_on_delete_redirect_url(
                    category, language)
                url = get_admin_url('aldryn_faq_category_delete',
                                    [category.pk, ])
                menu.add_modal_item(_('Delete this category'), url=url,
                                    on_close=redirect_url)

            if any(config_perms + category_perms) and any(question_perms):
                menu.add_break()

            # ------ Question items -------------------------------------------

            if add_question_perm:
                url_args = {'appconfig': self.config.pk, }
                if language:
                    url_args.update({'language': language, })
                if category:
                    url_args.update({'category': category.pk, })
                url = get_admin_url('aldryn_faq_question_add', **url_args)
                menu.add_modal_item(_('Add new question'), url=url)

            if change_question_perm and question:
                url_args = {}
                if language:
                    url_args = {'language': language, }
                url = get_admin_url('aldryn_faq_question_change',
                                    [question.pk, ], **url_args)
                menu.add_modal_item(_('Edit this question'), url=url,
                                    active=True)

            if delete_question_perm and question:
                redirect_url = self.get_on_delete_redirect_url(
                    question, language)
                url = get_admin_url('aldryn_faq_question_delete',
                                    [question.pk, ])
                menu.add_modal_item(_('Delete this question'), url=url,
                                    on_close=redirect_url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0005_auto_20150720_1344'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='faqconfig',
            options={'verbose_name': 'config', 'verbose_name_plural': 'configs'},
        ),
        migrations.AlterModelOptions(
            name='faqconfigtranslation',
            options={'default_permissions': (), 'verbose_name': 'config Translation', 'managed': True},
        ),
        migrations.AddField(
            model_name='questiontranslation',
            name='slug',
            field=models.SlugField(help_text='Provide a "slug" for this category or leave blank for an auto-generated one.', max_length=255, verbose_name='Slug', blank=True),
        ),
        migrations.AlterField(
            model_name='categorytranslation',
            name='slug',
            field=models.SlugField(help_text='Provide a "slug" for this category or leave blank for an auto-generated one.', max_length=255, verbose_name='Slug', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test.utils import override_settings
from cms.utils.urlutils import admin_reverse
from djangocms_helper.utils import create_user

from .test_base import AldrynFaqTest


# session engine is hardcoded in djangocms-helper (atm v0.9.4), so override
# per test case
@override_settings(SESSION_ENGINE='django.contrib.sessions.backends.cached_db')
class AdminViewsTestCase(AldrynFaqTest):
    tag_html = '<p>no html</p>'
    escaped_tag_html = '&lt;p&gt;no html&lt;/p&gt;'

    def setUp(self):
        super(AdminViewsTestCase, self).setUp()
        username = 'admin_user'
        password = 'test'
        self.admin_user = create_user(
            username=username,
            email='test@example.com',
            password=password,
            is_superuser=True,
        )
        self.client.login(username=username, password=password)

    def _test_admin_view(self, view_name, args=None):
        view_url = admin_reverse(view_name, args=args)
        response = self.client.get(view_url)
        # ensure that html was escaped
        self.assertNotContains(response, self.tag_html)
        self.assertContains(response, self.escaped_tag_html)

    def test_admin_add_veiw(self):
        view_name = 'aldryn_faq_question_change'
        self.question1.tags.add(self.tag_html)
        self._test_admin_view(view_name, args=[self.question1.pk])

    def test_admin_changelist_veiw(self):
        view_name = 'aldryn_faq_question_changelist'
        self.question1.tags.add(self.tag_html)
        self._test_admin_view(view_name)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import NoReverseMatch
from django.utils.translation import (
    get_language_from_request,
    ugettext_lazy as _,
)

from cms.menu_bases import CMSAttachMenu
from cms.apphook_pool import apphook_pool
from menus.base import NavigationNode
from menus.menu_pool import menu_pool

from .models import Category


class FaqCategoryMenu(CMSAttachMenu):
    name = _('FAQ')

    def get_nodes(self, request):
        nodes = []
        language = get_language_from_request(request, check_path=True)
        # don't bother with categories that don't have appconfig.
        categories = Category.objects.active_translations(
            language_code=language).exclude(appconfig__isnull=True).distinct()
        if hasattr(self, 'instance') and self.instance:
            #
            # If self has a property `instance`
            # then we're using django CMS 3.0.12 or later,
            # which supports using CMSAttachMenus on multiple,
            # apphook'ed pages, each with their own apphook configuration. So,
            # here we modify the queryset to reflect this.
            #
            app = apphook_pool.get_apphook(self.instance.application_urls)
            config = app.get_config(self.instance.application_namespace)
            if config:
                categories = categories.filter(appconfig=config)
        for category in categories:
            try:
                url = category.get_absolute_url(language=language)
            except NoReverseMatch:
                url = None

            if url:
                node = NavigationNode(
                    category.safe_translation_getter(
                        'name', language_code=language),
                    url,
                    category.pk,
                )
                nodes.append(node)
                for question in category.questions.all():
                    try:
                        q_url = question.get_absolute_url(language=language)
                    except NoReverseMatch:
                        q_url = None

                    if q_url:
                        node = NavigationNode(
                            question.safe_translation_getter(
                                'title', language_code=language),
                            q_url,
                            # NOTE: We're adding 1 million here to avoid
                            # clashing with the category IDs.
                            category.pk * 1000000 + question.pk,
                            category.pk,
                        )
                        nodes.append(node)

        return nodes

menu_pool.register_menu(FaqCategoryMenu)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0003_auto_20140926_2347'),
        ('aldryn_faq', '0011_create_default_faq_config'),
    ]

    operations = [
        migrations.AddField(
            model_name='categorytranslation',
            name='description',
            field=djangocms_text_ckeditor.fields.HTMLField(default='', help_text='Optional. Description of this category.', verbose_name='description', blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='selectedcategory',
            name='position',
            field=models.PositiveIntegerField(default=0, null=True, verbose_name='position', blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import NoReverseMatch
from django.utils.translation import override

from .test_base import AldrynFaqTest


class TestGetAbsoluteUrls(AldrynFaqTest):

    def test_category_urls(self):
        category_1_pk = self.category1.pk
        category_2_pk = self.category2.pk

        with override('en'):
            category_1_url = self.category1.get_absolute_url()
            category_2_url = self.category2.get_absolute_url()

        self.assertEquals('/en/faq/{cat}-example/'.format(
            cat=category_1_pk), category_1_url)
        # category2 doesn't exist EN, so this should fallback to DE
        self.assertEquals('/en/faq/{cat}-beispiel2/'.format(
            cat=category_2_pk), category_2_url)

        with override('de'):
            category_1_url = self.category1.get_absolute_url()
            category_2_url = self.category2.get_absolute_url()

        self.assertEquals('/de/faq/{cat}-beispiel/'.format(
            cat=category_1_pk), category_1_url)
        self.assertEquals('/de/faq/{cat}-beispiel2/'.format(
            cat=category_2_pk), category_2_url)

        # test that we can override the context with the language parameter
        with override('en'):
            category_1_url = self.category1.get_absolute_url(language="de")
            category_2_url = self.category2.get_absolute_url(language="de")

        self.assertEquals('/de/faq/{cat}-beispiel/'.format(
            cat=category_1_pk), category_1_url)
        self.assertEquals('/de/faq/{cat}-beispiel2/'.format(
            cat=category_2_pk), category_2_url)

        # For completeness, do the other way too
        with override('de'):
            category_1_url = self.category1.get_absolute_url(language="en")
            category_2_url = self.category2.get_absolute_url(language="en")

        self.assertEquals('/en/faq/{cat}-example/'.format(
            cat=category_1_pk), category_1_url)
        # category2 doesn't exist EN, so this should fallback to DE
        self.assertEquals('/en/faq/{cat}-beispiel2/'.format(
            cat=category_2_pk), category_2_url)

    def test_category_urls_fallbacks(self):
        category_1 = self.category1
        category_2 = self.category2

        # category 2 is not translated in english
        # so given our test fallback config
        # this should fallback to the german category
        self.assertEqual(
            category_2.get_absolute_url("en"),
            "/en/faq/{cat}-beispiel2/".format(cat=category_2.pk)
        )

        # category 1 is not translated in french
        # so given our test fallback config
        # this should fallback to the english category
        self.assertEqual(
            category_1.get_absolute_url("fr"),
            "/fr/faq/{cat}-example/".format(cat=category_1.pk)
        )

        with self.assertRaises(NoReverseMatch):
            # this should raise a NoRerverseMatch error
            # because category 2 is not translated in french
            # so it falls back to english which also does not exist
            category_2.get_absolute_url("fr")

    def test_question_urls(self):
        question_1_pk = self.question1.pk
        question_1_category_pk = self.question1.category_id

        question_2_pk = self.question2.pk
        question_2_category_pk = self.question2.category_id

        with override('en'):
            question_1 = self.reload(self.question1)
            question_1_url = question_1.get_absolute_url()

            question_2 = self.reload(self.question2)

            question_2_url = question_2.get_absolute_url()

        self.assertEquals('/en/faq/{cat_pk}-example/{pk}/'.format(
            cat_pk=question_1_category_pk, pk=question_1_pk), question_1_url)

        self.assertEquals('/en/faq/{cat_pk}-beispiel2/{pk}/'.format(
            cat_pk=question_2_category_pk, pk=question_2_pk), question_2_url)

        with override('de'):
            question_1 = self.reload(self.question1)
            question_1_url = question_1.get_absolute_url()

            question_2 = self.reload(self.question2)
            question_2_url = question_2.get_absolute_url()

        self.assertEquals('/de/faq/{cat_pk}-beispiel/{pk}/'.format(
            cat_pk=question_1_category_pk, pk=question_1_pk), question_1_url)

        self.assertEquals('/de/faq/{cat_pk}-beispiel2/{pk}/'.format(
            cat_pk=question_2_category_pk, pk=question_2_pk), question_2_url)

    def test_question_urls_fallbacks(self):
        question_1 = self.question1
        question_1_pk = question_1.pk
        question_1_category_pk = question_1.category_id

        question_2 = self.question2
        question_2_pk = question_2.pk
        question_2_category_pk = question_2.category_id

        # question 2 is not translated in english
        # so given our test fallback config
        # this should fallback to the german category
        self.assertEqual(
            question_2.get_absolute_url("en"),
            "/en/faq/{cat_pk}-beispiel2/{pk}/".format(
                cat_pk=question_2_category_pk, pk=question_2_pk)
        )

        # question 1 is not translated in french
        # so given our test fallback config
        # this should fallback to the english category
        self.assertEqual(
            question_1.get_absolute_url("fr"),
            "/fr/faq/{cat_pk}-example/{pk}/".format(
                cat_pk=question_1_category_pk, pk=question_1_pk)
        )

        with self.assertRaises(NoReverseMatch):
            # this should raise a NoRerverseMatch error
            # because category 2 is not translated in french
            # so it falls back to english which also does not exist
            question_2.get_absolute_url("fr")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def preset_order(apps, schema_editor):
    """Set initial ordering of category list plugin categories."""
    SelectedCategory = apps.get_model("aldryn_faq", "SelectedCategory")
    order = 0
    for obj in SelectedCategory.objects.all():
        order += 1
        obj.position = order
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0002_default_position'),
    ]

    operations = [
        migrations.RunPython(preset_order),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-24 02:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0012_categorytranslation_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='latestquestionsplugin',
            name='cache_duration',
            field=models.PositiveSmallIntegerField(default=0, help_text="The maximum duration (in seconds) that this plugin's content should be cached."),
        ),
        migrations.AddField(
            model_name='mostreadquestionsplugin',
            name='cache_duration',
            field=models.PositiveSmallIntegerField(default=0, help_text="The maximum duration (in seconds) that this plugin's content should be cached."),
        ),
        migrations.AddField(
            model_name='topquestionsplugin',
            name='cache_duration',
            field=models.PositiveSmallIntegerField(default=0, help_text="The maximum duration (in seconds) that this plugin's content should be cached."),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import resolve, reverse
from django.db import models
from django.http import (
    Http404,
    HttpResponseRedirect,
    HttpResponsePermanentRedirect,
)
from django.utils.translation import (
    get_language_from_request,
    override as force_language,
    ugettext,
)
from django.views.generic import DetailView
from django.views.generic.list import ListView

from aldryn_apphooks_config.mixins import AppConfigMixin
from parler.views import FallbackLanguageResolved, TranslatableSlugMixin
from parler.utils import get_active_language_choices
from menus.utils import set_language_changer

from .models import Category, Question

from . import request_faq_category_identifier, request_faq_question_identifier


# TODO: Move this to Aldryn Translation Tools
class AllowPKsTooMixin(object):
    def get_object(self, queryset=None):
        """
        Bypass TranslatableSlugMixin if we are using PKs. You would only use
        this if you have a obj that supports accessing the object by pk or
        by its translatable slug.

        NOTE: This should only be used on DetailViews and this mixin MUST be
        placed to the left of TranslatableSlugMixin. In fact, for best results,
        declare your obj like this:

            MyView(…, AllowPKsTooMixin, TranslatableSlugMixin, DetailView):
        """
        if self.pk_url_kwarg in self.kwargs:
            return super(DetailView, self).get_object(queryset)

        # OK, just let Parler have its way with it.
        return super(AllowPKsTooMixin, self).get_object(queryset)


# TODO: Move this to Aldryn Translation Tools
class CanonicalUrlMixin(object):
    """
    Provides configurable control over how non-canonical URLs to views are
    handled. A view can specify by setting 'non_canonical_url_response_type' to
    one of 200, 301, 302 or 404. By default, handling will be to temporarily
    redirect to the canonical URL.
    """
    non_canonical_url_response_type = 302

    def get_non_canonical_url_response_type(self):
        response_type = getattr(self, "non_canonical_url_response_type", None)
        if response_type and response_type in [200, 301, 302, 404]:
            return response_type
        else:
            return self.non_canonical_url_response_type

    def get(self, request, *args, **kwargs):
        """
        On GET, if the URL used is not the correct one, handle according to
        preferences by either:
            Allowing (200),
            Temporarily redirecting (302),
            Permanently redirecting (301) or
            Failing (404).
        """
        if not hasattr(self, 'object'):
            self.object = self.get_object()
        url = self.object.get_absolute_url()
        response_type = self.get_non_canonical_url_response_type()
        if response_type == 200 or request.path == url:
            return super(CanonicalUrlMixin, self).get(
                request, *args, **kwargs)
        if response_type == 302:
            return HttpResponseRedirect(url)
        elif response_type == 301:
            return HttpResponsePermanentRedirect(url)
        else:
            raise Http404('This is not the canonical uri of this object.')


class FaqCategoryMixin(AppConfigMixin):
    """
    Provides support for getting the category from the URL, even if the view
    using it is a DetailView for a different model, or not a DetailView at all.
    """
    category_pk_url_kwarg = 'category_pk'
    category_slug_url_kwarg = 'category_slug'

    def dispatch(self, request, *args, **kwargs):
        self.current_language = get_language_from_request(
            request, check_path=True)
        return super(FaqCategoryMixin, self).dispatch(
            request, *args, **kwargs)

    def get_language_choices(self):
        """
        Define the language choices for the view, defaults to the defined
        settings.
        """
        return get_active_language_choices(self.current_language)

    def get_category(self, queryset=None):
        """
        Fetch the object using a translated slug. This is largely stolen from
        Parler, but modified to remove any assumptions that this is being used
        on a DetailView.
        """
        if queryset is None:
            queryset = self.get_category_queryset()

        slug = self.kwargs.get(self.category_slug_url_kwarg, None)
        pk = self.kwargs.get(self.category_pk_url_kwarg, None)
        choices = self.get_language_choices()

        error_message = ugettext(
            "No %(verbose_name)s found matching the query") % {
                'verbose_name': queryset.model._meta.verbose_name}

        if pk:
            try:
                obj = Category.objects.get(pk=pk)
            except ObjectDoesNotExist:
                raise Http404(error_message)
        elif slug:
            obj = None
            using_fallback = False
            prev_choices = []
            for lang_choice in choices:
                try:
                    # Get the single item from the filtered queryset
                    # NOTE. Explicitly set language to the state the object was
                    # fetched in.
                    filters = {'slug': slug}
                    obj = queryset.translated(
                        lang_choice, **filters).language(lang_choice).get()
                except ObjectDoesNotExist:
                    # Translated object not found, next object is marked as
                    # fallback.
                    using_fallback = True
                    prev_choices.append(lang_choice)
                else:
                    break

            if obj is None:
                tried_msg = ", tried languages: {0}".format(", ".join(choices))
                raise Http404(error_message + tried_msg)

            # Object found!
            if using_fallback:
                for prev_choice in prev_choices:
                    if obj.has_translation(prev_choice):
                        raise FallbackLanguageResolved(obj, prev_choice)

        else:
            raise Http404(error_message)

        return obj

    def get_category_queryset(self):
        return Category.objects.language(language_code=self.current_language)

    def handle_old_url_exception(self, error):
        return HttpResponsePermanentRedirect(error.new_url_format)


class FaqAnswerView(CanonicalUrlMixin, FaqCategoryMixin, AllowPKsTooMixin,
                    TranslatableSlugMixin, DetailView):
    template_name = 'aldryn_faq/question_detail.html'
    model = Question

    def get_non_canonical_url_response_type(self):
        if not hasattr(self, 'object'):
            self.object = self.get_object()

        try:
            return self.object.category.appconfig.non_permalink_handling
        except AttributeError:
            return super(
                FaqAnswerView, self).get_non_canonical_url_response_type()

    def get(self, request, *args, **kwargs):
        try:
            category = self.get_category()
        except FallbackLanguageResolved as flr:
            # We have the category, but it is in a fallback language.
            category = flr.object

        # only look at questions within this category
        queryset = self.get_queryset().filter(category=category.pk)
        question = self.get_object(queryset=queryset)
        set_language_changer(request, question.get_absolute_url)

        if hasattr(request, 'toolbar'):
            request.toolbar.set_object(question)

        setattr(request, request_faq_category_identifier, question.category)
        setattr(request, request_faq_question_identifier, question)
        response = super(FaqAnswerView, self).get(request, *args, **kwargs)

        # FIXME: We should check for unique visitors using sessions.
        # update number of visits
        question_only_queryset = self.get_queryset().filter(pk=question.pk)
        question_only_queryset.update(
            number_of_visits=models.F('number_of_visits') + 1)
        return response

    def get_category_url(self):
        if not hasattr(self, 'object'):
            self.object = self.get_object()
        category = self.object.category
        return category.get_absolute_url(self.current_language)

    def get_context_data(self, **kwargs):
        context = super(FaqAnswerView, self).get_context_data(**kwargs)
        context['category_url'] = self.get_category_url()
        return context

    def get_object(self, queryset=None):
        if not hasattr(self, 'object'):
            # this is done because this method gets called twice.
            # so no need to query db twice.
            self.object = super(FaqAnswerView, self).get_object(queryset)
        return self.object

    def handle_old_url_exception(self, error):
        match = resolve(error.new_url_format)

        kwargs = match.kwargs
        kwargs['pk'] = self.kwargs['pk']
        url_name = '{0}:faq-answer'.format(match.namespace)

        with force_language(self.current_language):
            new_url_format = reverse(url_name, kwargs=kwargs)
        return HttpResponsePermanentRedirect(new_url_format)


class FaqByCategoryView(CanonicalUrlMixin, FaqCategoryMixin, ListView):
    template_name = 'aldryn_faq/question_list.html'
    model = Question

    def get_non_canonical_url_response_type(self):
        if not hasattr(self, 'object'):
            self.object = self.get_object()

        try:
            return self.object.appconfig.non_permalink_handling
        except AttributeError:
            return super(
                FaqByCategoryView, self).get_non_canonical_url_response_type()

    def get(self, request, *args, **kwargs):
        category = self.get_category()
        self.category = category

        setattr(request, request_faq_category_identifier, self.category)
        set_language_changer(request, self.category.get_absolute_url)
        response = super(FaqByCategoryView, self).get(request, *args, **kwargs)
        return response

    def get_object(self):
        return super(FaqByCategoryView, self).get_category()

    def get_context_data(self, **kwargs):
        kwargs['category_list'] = (
            self.get_category_queryset().active_translations(
                self.current_language).filter(appconfig=self.config))
        kwargs['active_category'] = self.get_category()
        return super(FaqByCategoryView, self).get_context_data(**kwargs)

    def get_queryset(self):
        queryset = super(FaqByCategoryView, self).get_queryset()
        # get questions with fallbacks
        queryset = queryset.active_translations(self.current_language)
        # only matching current category
        queryset = queryset.filter(category=self.category).order_by('order')
        return queryset


class FaqQuestionMixin(AppConfigMixin):
    model = Question

    def dispatch(self, request, *args, **kwargs):
        """Determines the current language from the request, stores it."""
        self.current_language = get_language_from_request(
            request, check_path=True)
        return super(FaqQuestionMixin, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(FaqQuestionMixin, self).get_context_data(**kwargs)
        context['current_app'] = self.namespace
        return context

    def get_queryset(self):
        queryset = self.model.objects.language(
            language_code=self.current_language
        ).select_related('category')
        return queryset


class FaqByCategoryListView(FaqCategoryMixin, ListView):
    template_name = 'aldryn_faq/landing.html'
    model = Category

    def get_queryset(self):
        return self.model.objects.language(
            language_code=self.current_language).active_translations(
                self.current_language).filter(appconfig=self.config)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from aldryn_apphooks_config.app_base import CMSConfigApp
from cms.apphook_pool import apphook_pool

from .models import FaqConfig


class FaqApp(CMSConfigApp):
    name = _('FAQ')
    urls = ['aldryn_faq.urls']
    app_name = 'aldryn_faq'
    app_config = FaqConfig

apphook_pool.register(FaqApp)



# -*- coding: utf-8 -*-
import sys

from distutils.version import LooseVersion

import cms

from djangocms_helper.utils import create_user

from .test_base import AldrynFaqTest


if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

# The CMS wizard system was introduced in 3.2.0
CMS_3_2 = LooseVersion(cms.__version__) >= LooseVersion('3.2.0')


@unittest.skipUnless(CMS_3_2, "No wizard support in CMS < 3.2")
class TestFAQWizard(AldrynFaqTest):

    def setUp(self):
        super(TestFAQWizard, self).setUp()
        username = 'admin_user'
        password = 'test'
        self.admin_user = create_user(
            username=username,
            email='test@example.com',
            password=password,
            is_superuser=True,
        )
        self.client.login(username=username, password=password)

    def test_question_wizard(self):
        # Import here to avoid logic wizard machinery
        from aldryn_faq.cms_wizards import CreateFaqQuestionForm

        data = {
            'title': 'Where are we?',
            'answer': 'Here.',
            'answer_text': '<p>Interesting question..</p>',
            'category': self.category1.pk,
        }
        form = CreateFaqQuestionForm(
            data=data,
            wizard_language='en',
            wizard_user=self.admin_user,
        )
        self.assertTrue(form.is_valid())
        question = form.save()

        url = question.get_absolute_url('en')
        response = self.client.get(url)
        self.assertContains(response, 'Where are we?', status_code=200)
        self.assertContains(response, 'Here.', status_code=200)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models, migrations, transaction
from django.apps import apps as django_apps
from django.db.utils import ProgrammingError, OperationalError

APP_PACKAGE = 'aldryn_faq'
APP_CONFIG = 'FaqConfig'
DEFAULT_NAMESPACE = 'aldryn_faq_default'
DEFAULT_APP_TITLE = 'Default FAQ'


def noop(apps, schema_editor):
    pass


def get_config_count(model_class):
    with transaction.atomic():
        count = model_class.objects.count()
    return count


def create_placeholders(app_config, save=True):
    """
    Creates placeholder instances for each Placeholder field on provided
    app_config.
    """
    import cms.models.fields
    from cms.models import Placeholder

    for field in app_config._meta.fields:
        if not field.__class__ == cms.models.fields.PlaceholderField:
            # skip other fields.
            continue
        placeholder_name = field.name
        placeholder_id_name = '{0}_id'.format(placeholder_name)
        placeholder_id = getattr(app_config, placeholder_id_name, None)
        if placeholder_id is not None:
            # do not process if it has a reference to placeholder field.
            continue
        # since there is no placeholder - create it, we cannot use
        # get_or_create because it can get placeholder from other config
        new_placeholder = Placeholder.objects.create(
            slot=field.slotname)
        setattr(app_config, placeholder_id_name, new_placeholder.pk)
    # after we process all placeholder fields - save config,
    # so that django can pick up them.
    if save:
        app_config.save()


def create_default_config(apps, schema_editor):
    FaqConfig = apps.get_model(APP_PACKAGE, APP_CONFIG)

    # if we try to execute this migration after cms migrations were migrated
    # to latest - we would get an exception because apps.get_model
    # contains cms models in the last known state (which is the dependency
    # migration state). If that is the case we need to import the real model.
    try:
        # to avoid the following error:
        #   django.db.utils.InternalError: current transaction is aborted,
        #   commands ignored until end of transaction block
        # we need to cleanup or avoid that by making transaction atomic.
        count = get_config_count(FaqConfig)
    except (ProgrammingError, OperationalError):
        model_path = '{0}.{1}'.format(APP_PACKAGE, APP_CONFIG)
        FaqConfig = django_apps.get_model(model_path)
        count = get_config_count(FaqConfig)

    if not count == 0:
        for cfg in FaqConfig.objects.all():
            create_placeholders(cfg)
        return
    # create only if there is no configs because user may already have
    # existing and configured config.
    app_config = FaqConfig(namespace=DEFAULT_NAMESPACE)
    # usually generated in aldryn_apphooks_config.models.AppHookConfig
    # but in migrations we don't have real class with correct parents.
    app_config.type = '{0}.cms_appconfig.{1}'.format(APP_PACKAGE, APP_CONFIG)
    # placeholders
    # cms checks if instance.pk is set, and if it isn't cms creates a new
    # placeholder but it does that with real models, and fields on instance
    # are faked models. To prevent that we need to manually set instance pk.
    app_config.pk = 1
    # placeholders
    create_placeholders(app_config, save=False)
    app_config.save()

    # translations
    app_config_translation = app_config.translations.create()
    app_config_translation.language_code = settings.LANGUAGES[0][0]
    app_config_translation.app_title = DEFAULT_APP_TITLE
    app_config_translation.save()


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0003_auto_20140926_2347'),
        ('aldryn_faq', '0010_auto_20160109_2144')
    ]

    operations = [
        migrations.RunPython(create_default_config, noop)
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from cms.utils.i18n import get_current_language
from parler.managers import TranslatableManager


class RelatedManager(TranslatableManager):

    def filter_by_language(self, language):
        return self.active_translations(language_code=language)

    def filter_by_current_language(self):
        return self.filter_by_language(get_current_language())


class CategoryManager(TranslatableManager):

    def get_categories(self, language=None):
        categories = self.active_translations(
            language_code=language).prefetch_related('questions')

        for category in categories:
            questions_by_language = category.questions.filter_by_language
            category.count = questions_by_language(language).count()
        return sorted(categories, key=lambda x: -x.count)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import FaqAnswerView, FaqByCategoryView, FaqByCategoryListView


urlpatterns = patterns('',
    url(r'^$', FaqByCategoryListView.as_view(), name='faq-category-list'),


    # 1/
    url(
        r'^(?P<category_pk>\d+)/$',
        FaqByCategoryView.as_view(),
        name='faq-category'
    ),
    # 1/2/
    url(
        r'^(?P<category_pk>\d+)/(?P<pk>\d+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),
    # 1/question-slug/
    url(
        r'^(?P<category_pk>\d+)/(?P<slug>[-\w]+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),

    # NOTE: This set must appear before the 'category-slug/*' set below

    # 1-category-slug/
    url(
        r'^(?P<category_pk>\d+)-(?P<category_slug>[-\w]+)/$',
        FaqByCategoryView.as_view(),
        name='faq-category'
    ),
    # 1-category-slug/2/
    url(
        r'^(?P<category_pk>\d+)-(?P<category_slug>[-\w]+)/(?P<pk>\d+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),
    # 1-category-slug/question-slug/
    url(
        r'^(?P<category_pk>\d+)-(?P<category_slug>[-\w]+)/(?P<slug>[-\w]+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),


    # category-slug/
    url(
        r'^(?P<category_slug>[-\w]+)/$',
        FaqByCategoryView.as_view(),
        name='faq-category'
    ),
    # category-slug/2/
    url(
        r'^(?P<category_slug>[-\w]+)/(?P<pk>\d+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),
    # category-slug/question-slug/
    url(
        r'^(?P<category_slug>[-\w]+)/(?P<slug>[-\w]+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import app_data.fields


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0009_auto_20160107_1043'),
    ]

    operations = [
        migrations.AlterField(
            model_name='faqconfig',
            name='app_data',
            field=app_data.fields.AppDataField(default='{}', editable=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

from django.utils.text import slugify
from django.utils.encoding import force_text

from aldryn_translation_tools.models import TranslatedAutoSlugifyMixin


class AutoSlugifyQuestions(TranslatedAutoSlugifyMixin):
    slug_max_length = 255
    slug_source_field_name = 'title'

    def __init__(self, translation, query_model=None):
        self.translation = translation
        self.query_model = query_model

    # override some methods since we are performing operaions on the translation
    # not the question itself.
    def get_slug_source(self):
        return getattr(self.translation, self.slug_source_field_name, None)

    def _get_existing_slug(self):
        getattr(self.translation, self.slug_field_name, None)

    def _get_slug_queryset(self, lookup_model=None):
        return super(AutoSlugifyQuestions, self)._get_slug_queryset(
            lookup_model=self.query_model)

    def get_current_language(self):
        return self.translation.language_code


def resave_questions_for_slug_autogeneration(apps, schema_editor):
    """Trigger save method for each existing Question to generate slug"""
    Question = apps.get_model("aldryn_faq", "Question")
    for question in Question.objects.all():
        for translation in question.translations.all():
            AutoSlugify = AutoSlugifyQuestions(translation)
            translation.slug = AutoSlugify.make_new_slug(
                qs=Question.objects.all())
            translation.save()


def do_nothing(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0007_auto_20150903_1503'),
    ]

    operations = [
        migrations.RunPython(resave_questions_for_slug_autogeneration, do_nothing),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from distutils.version import LooseVersion

from django.contrib import admin
from django.templatetags.static import static
from django.utils.translation import ugettext as _
from django.utils.html import escape

import cms
from cms.admin.placeholderadmin import FrontendEditableAdminMixin

from adminsortable2.admin import SortableAdminMixin
from aldryn_apphooks_config.admin import BaseAppHookConfig
from aldryn_reversion.admin import VersionedPlaceholderAdminMixin
from aldryn_translation_tools.admin import AllTranslationsMixin
from parler.admin import TranslatableAdmin

from .models import Category, Question, FaqConfig
from .forms import CategoryAdminForm


class CategoryAdmin(AllTranslationsMixin,
                    VersionedPlaceholderAdminMixin,
                    TranslatableAdmin):

    list_display = ('__str__', 'appconfig', )

    form = CategoryAdminForm

    _fieldsets = [
        (None, {
            'fields': ('name', 'slug', )
        }),
        (_('Language Independent Fields'), {
            'fields': ('appconfig', )
        }),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': ('description',)
        }),
    ]

    class Media:
        # Workaround for known Django bug: #24467
        # https://code.djangoproject.com/ticket/24467
        # Django checks for self.prepopulated_fields to determine if it should
        # include populate.js. But it never checks get_prepopulated_fields().
        # Fix is slated for Django 1.9 release, so this needs to remain until
        # we no longer support Django 1.8 or lower.
        js = [static('admin/js/%s' % url) for url in (
            'urlify.js', 'prepopulate.min.js')]

    def get_prepopulated_fields(self, request, obj=None):
        return {'slug': ['name']}

    def get_fieldsets(self, request, obj=None):
        return self._fieldsets


class QuestionAdmin(AllTranslationsMixin,
                    VersionedPlaceholderAdminMixin,
                    FrontendEditableAdminMixin,
                    SortableAdminMixin,
                    TranslatableAdmin):

    render_placeholder_language_tabs = False
    list_display = [
        '__str__', 'category', 'tag_list', 'is_top', 'number_of_visits']
    list_filter = ['category', 'translations__language_code']
    frontend_editable_fields = ('title', 'category', 'answer_text')
    readonly_fields = ['number_of_visits']

    def tag_list(self, obj):
        """
        Displays Taggit tags to a comma-separated list of the tags’ names.
        """
        escaped_tags = [escape(tag.name) for tag in obj.tags.get_query_set()]
        return ", ".join(escaped_tags)
    tag_list.short_description = 'Tags'
    tag_list.allow_tags = True

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (None, {
                'fields': [
                    'title', 'slug', 'category', 'answer_text', 'tags',
                    'is_top', 'number_of_visits']
            })
        ]
        cms_compat_fieldset = {
            'classes': ['plugin-holder', 'plugin-holder-nopage'],
            'fields': ['answer']
        }

        # show placeholder field if not CMS 3.0
        if LooseVersion(cms.__version__) < LooseVersion('3.0'):
            fieldsets.append(('Short description', cms_compat_fieldset))
        return fieldsets


admin.site.register(Category, CategoryAdmin)
admin.site.register(Question, QuestionAdmin)


class FaqConfigAdmin(AllTranslationsMixin,
                     VersionedPlaceholderAdminMixin,
                     BaseAppHookConfig,
                     TranslatableAdmin):
    def get_config_fields(self):
        return (
            'app_title',
            'permalink_type', 'non_permalink_handling',
            'config.show_description',
        )

admin.site.register(FaqConfig, FaqConfigAdmin)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

try:
    from reversion.revisions import create_revision, get_for_object
except ImportError:
    from reversion import create_revision, get_for_object

from django.db import transaction

from ..models import Category, Question, FaqConfig

from .test_base import AldrynFaqTest


class CommonAldrynReversionsAdminMixinTestCaseMixin(object):
    """
    Provides a set of utility methods and test cases to check if model was
    registered correctly with VersionnedPlaceholderAdminMixin from
    aldrun-reversions.
    The test is to perform reversing of recovery url if that is not failed with
    exception, models is considered to be registered correctly for django admin.

    Class attributes:
        model_class: class of a model to test against
        text_fields_to_change: list of text fields to change with random value
                               for generating reversion
        object_instance: instance of model_class which is already created and
                         can be used for tests.
    """
    model_class = None
    text_fields_to_change = []
    object_instance = None

    def setUp(self):
        super(CommonAldrynReversionsAdminMixinTestCaseMixin, self).setUp()
        self.create_revision_dummy()

    def create_revision_dummy(self):
        """
        Just create a revision with dummy data to ensure we have at least
        one revision.
        :return: None
        """
        object_instance = self.get_object_instance()
        for field in self.text_fields_to_change:
            setattr(object_instance, field, self.rand_str(prefix='dummy_'))
        with transaction.atomic():
            with create_revision():
                object_instance.save()

    def get_object_instance(self):
        """
        Should return model instance.
        Ensure that it returns model instance before using this mixin.
        Or set up self.object_instance during setUp method.
        """
        return self.object_instance

    def test_category_recovery_accessible(self):
        object_instance = self.get_object_instance()
        version = get_for_object(object_instance)[0]
        object_pk = object_instance.pk
        self.assertEqual(self.model_class.objects.filter(
            pk=object_pk).count(), 1)
        object_instance.delete()
        self.assertEqual(self.model_class.objects.filter(
            pk=object_pk).count(), 0)

        # check that there is a a way to access recovery view
        obj = version.object_version.object
        opts = obj._meta
        url = reverse(
            'admin:{0}_{1}_{2}'.format(
                opts.app_label,
                obj._meta.model_name,
                'recover'),
            args=[version.pk])
        # ust in case check the length, but at this step either a
        # NoReverseMatch should occur or other error,
        # if no exception is raised, it is a good sign
        self.assertGreater(len(url), 4)


class CategoryReversionAdminTestCase(
    CommonAldrynReversionsAdminMixinTestCaseMixin,
        AldrynFaqTest):

    model_class = Category
    text_fields_to_change = ['name', 'slug']

    def get_object_instance(self):
        return self.category1


class QuestionReversionAdminTestCase(
    CommonAldrynReversionsAdminMixinTestCaseMixin,
        AldrynFaqTest):

    model_class = Question
    text_fields_to_change = ['title', 'slug']

    def get_object_instance(self):
        return self.question1


class ConfigReversionAdminTestCase(
    CommonAldrynReversionsAdminMixinTestCaseMixin,
        AldrynFaqTest):

    model_class = FaqConfig
    text_fields_to_change = ['app_title']

    def get_object_instance(self):
        return self.app_config



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_faq.menu import FaqCategoryMenu
from menus.menu_pool import menu_pool

from .test_base import AldrynFaqTest


class TestMenu(AldrynFaqTest):

    def test_get_nodes(self):
        # Test that the EN version of the menu has only category1 and its
        # question1, and is shown in English.
        request = self.get_page_request(
            None, self.user, path='/en/', lang_code='en')
        try:
            renderer = menu_pool.get_renderer(request)
            menu = FaqCategoryMenu(renderer=renderer)
        except AttributeError:
            menu = FaqCategoryMenu()
        category1 = self.reload(self.category1, 'en')
        question1 = self.reload(self.question1, 'en')
        self.assertEqualItems(
            [menuitem.title for menuitem in menu.get_nodes(request)],
            [category1.name, question1.title]
        )

        # Test that the DE version has 2 categories and their questions that
        # they are shown in German.
        request = self.get_page_request(
            None, self.user, path='/de/', lang_code='de')
        try:
            renderer = menu_pool.get_renderer(request)
            menu = FaqCategoryMenu(renderer=renderer)
        except AttributeError:
            menu = FaqCategoryMenu()
        nodes = menu.get_nodes(request)
        self.assertEqualItems(
            [menuitem.title for menuitem in nodes],
            [self.category1.name, self.category2.name, self.question1.title,
                self.question2.title]
        )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import six

from django.template import RequestContext
from django.utils.translation import override

from cms.api import add_plugin

from aldryn_faq.models import SelectedCategory
from .test_base import AldrynFaqTest

# NOTICE:
#
# For unknown reasons, in Django 1.8, this statement:
#
#     context = RequestContext(request, {})
#
# Seems to create a context that does not contain a request object, but only
# sometimes. This issue does not seem to exist in earlier versions of Django.
# Investigating further.
#
# The fix for now is to explicitly add another context item for the request
# object like so:
#
#     context = RequestContext(request, {"request": request})
#
# As required.


class TestQuestionListPlugin(AldrynFaqTest):

    def test_plugin(self):
        page1 = self.get_or_create_page("Page One")
        ph = page1.placeholders.get(slot="content")
        plugin = add_plugin(ph, "QuestionListPlugin", language="en")

        # First test that it is initially empty
        request = self.get_page_request(
            page1, self.user, None, lang_code="en", edit=False)
        context = RequestContext(request, {})
        rendered = plugin.render_plugin(context, ph)
        self.assertTrue(rendered.find("No entry found.") > -1)

        # Now, add a question, and test that it renders.
        question1 = self.reload(self.question1, "en")
        plugin.questions.add(question1)
        plugin.save()
        request = self.get_page_request(
            page1, self.user, None, lang_code="en", edit=False)
        context = RequestContext(request, {"request": request})
        rendered = plugin.render_plugin(context, ph)
        self.assertTrue(rendered.find(question1.title) > -1)

        # Test its unicode method
        self.assertEqual(str(plugin), "1 question selected")

        # Test its copy_relations. To do this, we'll create another instance
        # that is empty, then copy_relations to it, and prove that it contains
        # questions.
        plugin2 = add_plugin(ph, "QuestionListPlugin", language="en")
        plugin2.copy_relations(plugin)
        self.assertTrue(plugin.get_questions(), plugin2.get_questions())


class TestLatestQuestionsPlugin(AldrynFaqTest):

    def test_plugin(self):
        with override("de"):
            page1 = self.get_or_create_page("Page One")
            ph = page1.placeholders.get(slot="content")
            plugin = add_plugin(ph, "LatestQuestionsPlugin", language="de")
            request = self.get_page_request(
                page1, self.user, None, lang_code="de", edit=False)
            context = RequestContext(request, {"request": request})
            url1 = self.reload(self.question1, "de").get_absolute_url()
            url2 = self.reload(self.question2, "de").get_absolute_url()
            rendered = plugin.render_plugin(context, ph)
        self.assertTrue(rendered.find(url1) > -1)
        self.assertTrue(rendered.find(url2) > -1)
        # Test that question2 appears before question1
        self.assertTrue(rendered.find(url2) < rendered.find(url1))


class TestTopQuestionsPlugin(AldrynFaqTest):

    def test_plugin(self):
        page1 = self.get_or_create_page("Page One")
        ph = page1.placeholders.get(slot="content")
        plugin = add_plugin(ph, "TopQuestionsPlugin", language="en")

        # First test that no plugins are found initially
        request = self.get_page_request(
            page1, self.user, None, lang_code="en", edit=False)
        context = RequestContext(request, {})
        rendered = plugin.render_plugin(context, ph)
        self.assertTrue(rendered.find("No entry found") > -1)

        # Now test, set a question to be "top", then test that it appears.
        self.question1.is_top = True
        self.question1.save()
        request = self.get_page_request(
            page1, self.user, None, lang_code="en", edit=False)
        context = RequestContext(request, {"request": request})
        question1 = self.reload(self.question1, "en")
        rendered = plugin.render_plugin(context, ph)
        self.assertTrue(rendered.find(question1.title) > -1)


class TestMostReadQuestionsPlugin(AldrynFaqTest):
    def test_plugin(self):
        # Prepare the questions...
        self.question1.number_of_visits = 5
        self.question1.save()
        self.question2.number_of_visits = 10
        self.question2.save()

        with override("de"):
            page1 = self.get_or_create_page("Page One")
            ph = page1.placeholders.get(slot="content")
            plugin = add_plugin(ph, "MostReadQuestionsPlugin", language="de")
            request = self.get_page_request(
                page1, self.user, None, lang_code="de", edit=False)
            context = RequestContext(request, {"request": request})
            url1 = self.reload(self.question1, "de").get_absolute_url()
            url2 = self.reload(self.question2, "de").get_absolute_url()
            rendered = plugin.render_plugin(context, ph)
        # Ensure both questions appear...
        self.assertTrue(rendered.find(url1) > -1)
        self.assertTrue(rendered.find(url2) > -1)
        # Test that question2 appears before question1
        self.assertTrue(rendered.find(url2) < rendered.find(url1))


class TestCategoryListPlugin(AldrynFaqTest):
    def test_plugin(self):
        page1 = self.get_or_create_page("Page One")
        ph = page1.placeholders.get(slot='content')
        plugin = add_plugin(ph, 'CategoryListPlugin', language="de")

        request = self.get_page_request(
            page1, self.user, None, lang_code="de", edit=False)
        context = RequestContext(request, {})
        category1 = self.category1
        category1.save()
        category2 = self.category2
        category2.save()
        url = category1.get_absolute_url(language="de")
        rendered = plugin.render_plugin(context, ph)
        self.assertFalse(rendered.find(url) > -1)

        # Add some selected categories
        categories = [self.category1, self.category2]
        sc = None
        for idx, category in enumerate(categories):
            sc = SelectedCategory(
                category=category, position=idx, cms_plugin=plugin)
            sc.save()
        self.assertEqualItems(
            [c.id for c in plugin.get_categories()],
            [c.id for c in categories]
        )

        # While we're here, let's test that SelectedCategory's __str__ works
        if six.PY2:
            self.assertEqual(unicode(sc), categories[-1].name)
        else:
            self.assertEqual(str(sc), categories[-1].name)

        # Test that copy_relations works
        plugin2 = add_plugin(ph, "CategoryListPlugin", language="de")
        plugin2.copy_relations(plugin)
        self.assertEqualItems(
            [c.id for c in plugin.get_categories()],
            [c.id for c in plugin2.get_categories()]
        )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_faq.search_indexes import QuestionIndex, CategoryIndex

from .test_base import AldrynFaqTest


class TestQuestionIndex(AldrynFaqTest):
    def test_get_title(self):
        idx_obj = QuestionIndex()
        question1 = self.reload(self.question1, "en")
        self.assertEqual(idx_obj.get_title(question1), question1.title)

    def test_get_index_queryset(self):
        idx_obj = QuestionIndex()
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("en")],
            [self.question1.id],
        )
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("de")],
            [self.question1.id, self.question2.id],
        )

    def test_get_search_data(self):
        idx_obj = QuestionIndex()
        question1 = self.reload(self.question1, "en")
        search_data = idx_obj.get_search_data(question1, "en", None)
        self.assertEqual(search_data, "Test Question Test Answer")

        # This isn't working, but I think it proves that the search index
        # doesn't actually work translatedly.
        # TODO: Fix search_indexes.py
        # question1 = self.reload(self.question1, "de")
        # search_data = idx_obj.get_search_data(question1, "de", None)
        # self.assertEqual(search_data, "Testfrage Test Antwort")


class TestCategoryIndex(AldrynFaqTest):
    def test_get_title(self):
        idx_obj = CategoryIndex()
        category1 = self.reload(self.category1, "en")
        self.assertEqual(idx_obj.get_title(category1), category1.name)

    def test_get_index_queryset(self):
        idx_obj = CategoryIndex()
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("en")],
            [self.category1.id],
        )
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("de")],
            [self.category1.id, self.category2.id],
        )

    def test_get_search_data(self):
        idx_obj = CategoryIndex()
        category1 = self.reload(self.category1, "en")
        search_data = idx_obj.get_search_data(category1, "en", None)
        self.assertEqual(search_data, category1.name)

        # This isn't working, but I think it proves that the search index
        # doesn't actually work translatedly.
        # TODO: Fix search_indexes.py
        # category1 = self.reload(self.category1, "de")
        # search_data = idx_obj.get_search_data(category1, "de", None)
        # self.assertEqual(search_data, "Beispiel")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_faq', '0008_generate_slugs_for_existing_questions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='categorytranslation',
            name='name',
            field=models.CharField(help_text='Provide the category\u2019s name', max_length=255, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='questiontranslation',
            name='title',
            field=models.CharField(help_text='This should be a short form of the question', max_length=255, verbose_name='Title'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from aldryn_apphooks_config.models import AppHookConfig
from aldryn_reversion.core import version_controlled_content
from cms.models.fields import PlaceholderField
from parler.models import TranslatableModel, TranslatedFields

# Different styles of slugs supported where 1 is the category PK and 2 is the
# question PK
PERMALINK_CHOICES = (
    ('Pp', _('1/2/', )),
    ('Ps', _('1/question-slug/', )),
    ('Sp', _('category-slug/2/', )),
    ('Ss', _('category-slug/question-slug/', )),  # Default
    ('Bp', _('1-category-slug/2/', )),
    ('Bs', _('1-category-slug/question-slug/', )),
)

NON_PERMALINK_HANDLING = (
    (200, _('Allow')),
    (302, _('Redirect to permalink (default)')),  # Default
    (301, _('Permanent redirect to permalink')),
    (404, _('Return 404: Not Found')),
)


@version_controlled_content()
class FaqConfig(TranslatableModel, AppHookConfig):
    """Adds some translatable, per-app-instance fields."""
    translations = TranslatedFields(
        app_title=models.CharField(_('application title'), max_length=234),
    )

    permalink_type = models.CharField(_('permalink type'), max_length=2,
        blank=False, choices=PERMALINK_CHOICES, default="Ss",
        help_text=_('Choose the style of urls to use from the examples. '
                    '(Note, all types are relative to apphook)'))

    non_permalink_handling = models.SmallIntegerField(
        _('non-permalink handling'),
        blank=False, default=302,
        choices=NON_PERMALINK_HANDLING,
        help_text=_('How to handle non-permalink urls?'))

    # Category List PHFs
    placeholder_faq_sidebar_top = PlaceholderField(
        'faq_sidebar_top', related_name='aldryn_faq_sidebar_top')

    placeholder_faq_sidebar_bottom = PlaceholderField(
        'faq_sidebar_bottom', related_name='aldryn_faq_sidebar_bottom')

    placeholder_faq_content = PlaceholderField(
        'faq_content', related_name='aldryn_faq_content')

    # Question list PHFs
    placeholder_faq_list_top = PlaceholderField(
        'faq_list_top', related_name='aldryn_faq_list_top')

    placeholder_faq_list_bottom = PlaceholderField(
        'faq_list_bottom', related_name='aldryn_faq_list_bottom')

    class Meta:
        verbose_name = 'config'
        verbose_name_plural = 'configs'

    def get_app_title(self):
        return getattr(self, 'app_title', _('untitled'))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import six

from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import override, ugettext_lazy as _, ungettext

from aldryn_reversion.core import version_controlled_content
from aldryn_translation_tools.models import (
    TranslationHelperMixin, TranslatedAutoSlugifyMixin)
from cms.models.fields import PlaceholderField
from cms.models.pluginmodel import CMSPlugin
from cms.utils.i18n import get_current_language

from djangocms_text_ckeditor.fields import HTMLField
from parler.models import TranslatableModel, TranslatedFields
from sortedm2m.fields import SortedManyToManyField
from taggit.managers import TaggableManager

from .cms_appconfig import FaqConfig
from .managers import CategoryManager, RelatedManager
from .utils import is_valid_namespace, is_valid_app_config


def get_translation(obj, language_code):
    """
    This is an adapter from django-hvad.utils.get_translation(), a function
    to django-parler.models.get_translation() (a model instance method).
    """
    if not obj or not hasattr(obj, "get_translation"):
        return None
    return obj.get_translation(language_code)


def get_slug_in_language(record, language):
    """This is an adapter from django-hvad's lazy_translation_getter."""
    if not record or not hasattr(record, "safe_translation_getter"):
        return None
    return record.safe_translation_getter(
        field="slug", language_code=language, default=None, )


def filter_question_qs(question_qs):
    """
    Filters provided question queryset to ensure that only apphooked
    namespaces are being used.
    :param question_qs: QuestionQueryset
    :return: filtered question_qs
    """
    app_configs = set()
    for question in question_qs.iterator():
        app_config = question.category.appconfig
        if (is_valid_app_config(app_config) and
                is_valid_namespace(app_config.namespace)):
            app_configs.add(app_config)
    return question_qs.filter(category__appconfig__in=app_configs)


@python_2_unicode_compatible
@version_controlled_content(follow=['appconfig'])
class Category(TranslatedAutoSlugifyMixin, TranslationHelperMixin,
               TranslatableModel):
    slug_source_field_name = 'name'

    translations = TranslatedFields(
        name=models.CharField(
            _('name'), max_length=255,
            help_text=_(u"Provide the category’s name")),
        slug=models.SlugField(
            verbose_name=_('Slug'), max_length=255, blank=True,
            help_text=_('Provide a "slug" for this category or leave blank for '
                        'an auto-generated one.')),
        description=HTMLField(
            verbose_name=_('description'), blank=True, default='',
            help_text=_('Optional. Description of this category.'))
    )
    appconfig = models.ForeignKey(
        FaqConfig, verbose_name=_('appconfig'), blank=True, null=True
    )
    objects = CategoryManager()

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    def __str__(self):
        pkstr = str(self.pk)
        if six.PY2:
            pkstr = six.u(pkstr)
        return self.safe_translation_getter('name', default=pkstr)

    def model_type_id(self):
        return ContentType.objects.get_for_model(self.__class__).id

    def get_absolute_url(self, language=None, slug=None):
        language = language or get_current_language()

        if not slug:
            slug = self.known_translation_getter(
                'slug', default=None, language_code=language)[0] or ''

        kwargs = {}
        try:
            permalink_type = self.appconfig.permalink_type
        except AttributeError:
            permalink_type = "Ss"

        if 'P' in permalink_type:
            kwargs.update({"category_pk": self.pk})
        elif 'S' in permalink_type:
            kwargs.update({"category_slug": slug})
        else:
            kwargs = {'category_pk': self.pk, 'category_slug': slug}

        if self.appconfig_id and self.appconfig.namespace:
            namespace = '{0}:'.format(self.appconfig.namespace)
        else:
            namespace = ''

        with override(language):
            return reverse('{0}faq-category'.format(namespace), kwargs=kwargs)


@python_2_unicode_compatible
@version_controlled_content(follow=['category'])
class Question(TranslatedAutoSlugifyMixin, TranslationHelperMixin,
               TranslatableModel):
    slug_source_field_name = 'title'

    translations = TranslatedFields(
        title=models.CharField(
            _('Title'), max_length=255,
            help_text=_(u"This should be a short form of the question")),
        answer_text=HTMLField(_('Short description')),
        slug=models.SlugField(
            verbose_name=_('Slug'), max_length=255, blank=True,
            help_text=_('Provide a "slug" for this category or leave blank for '
                        'an auto-generated one.')),
    )
    category = models.ForeignKey(Category, related_name='questions')

    answer = PlaceholderField(
        'faq_question_answer', related_name='faq_questions')
    is_top = models.BooleanField(default=False)
    number_of_visits = models.PositiveIntegerField(default=0, editable=False)

    tags = TaggableManager(blank=True)

    objects = RelatedManager()

    order = models.PositiveIntegerField(default=1, db_index=True)

    class Meta:
        verbose_name = _('question')
        verbose_name_plural = _('questions')
        ordering = ('order', )

    def __str__(self):
        pkstr = str(self.pk)
        if six.PY2:
            pkstr = six.u(pkstr)
        return self.safe_translation_getter('title', default=pkstr)

    def model_type_id(self):
        return ContentType.objects.get_for_model(self.__class__).id

    def get_absolute_url(self, language=None):
        """
        Returns the absolute_url of this question object, respecting the
        configured fallback languages.
        """
        language = language or get_current_language()

        category_slug = self.category.known_translation_getter(
            'slug', default='', language_code=language)[0]

        question_slug = self.known_translation_getter(
            'slug', default='', language_code=language)[0]

        try:
            namespace = self.category.appconfig.namespace
        except AttributeError:
            namespace = False

        permalink_type = self.category.appconfig.permalink_type

        kwargs = {}
        if 'P' in permalink_type:
            kwargs.update({"category_pk": self.category.pk})
        elif 'S' in permalink_type:
            kwargs.update({"category_slug": category_slug})
        else:
            kwargs = {
                'category_pk': self.category.pk,
                'category_slug': category_slug
            }
        if 'p' in permalink_type:
            kwargs.update({"pk": self.pk})
        else:
            kwargs.update({"slug": question_slug})

        if namespace and category_slug:
            with override(language):
                url_name = '{0}:faq-answer'.format(namespace)
                return reverse(url_name, kwargs=kwargs)

        # No suitable translation exists, return the category's url
        return self.category.get_absolute_url(language)


class QuestionsPlugin(models.Model):
    questions = models.IntegerField(
        default=5,
        help_text=_('The number of questions to be displayed.')
    )

    def get_queryset(self):
        qs = filter_question_qs(
            Question.objects.filter_by_language(self.language))
        return qs

    def get_questions(self):
        questions = self.get_queryset()
        return questions[:self.questions]

    class Meta:
        abstract = True


@python_2_unicode_compatible
class QuestionListPlugin(CMSPlugin):
    questions = SortedManyToManyField(Question)

    def copy_relations(self, oldinstance):
        self.questions = oldinstance.questions.all()

    def get_questions(self):
        qs = filter_question_qs(self.questions.all())
        return qs

    def __str__(self):
        question_count = self.questions.count()
        return ungettext(
            "%(count)d question selected",
            "%(count)d questions selected",
            question_count
        ) % {"count": question_count, }


class CategoryListPlugin(CMSPlugin):

    def copy_relations(self, oldinstance):
        for category in oldinstance.selected_categories.all():
            category.pk = None
            category.cms_plugin = self
            category.save()

    def get_categories(self):
        """
        By default, if no categories were chosen return all categories.
        Otherwise, return the chosen categories.
        """
        # ensure we don't try to resolve categories we cannot resolve
        categories = [
            category for category in Category.objects.get_categories(
                language=self.language)
            if is_valid_app_config(category.appconfig) and is_valid_namespace(
                category.appconfig.namespace)]

        if self.selected_categories.exists():
            category_ids = self.selected_categories.values_list(
                'category__pk', flat=True)
            # categories is a list, and a sorted one so no need for another db
            # call.
            selected_categories = []
            for id in category_ids:
                category = next((x for x in categories if x.pk == id), None)
                if category:
                    selected_categories.append(category)
            return selected_categories
        return categories


@python_2_unicode_compatible
class SelectedCategory(models.Model):
    category = models.ForeignKey(to=Category, verbose_name=_('category'))
    position = models.PositiveIntegerField(
        verbose_name=_('position'), blank=True, default=0, null=True)
    cms_plugin = models.ForeignKey(
        to=CategoryListPlugin, related_name='selected_categories')

    class Meta:
        ordering = ['position', ]
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.category.name


class AdjustableCacheModelMixin(models.Model):
    # NOTE: This field shouldn't even be displayed in the plugin's change form
    # if using django CMS < 3.3.0
    cache_duration = models.PositiveSmallIntegerField(
        default=0,  # not the most sensible, but consistent with older versions
        blank=False,
        help_text=_(
            "The maximum duration (in seconds) that this plugin's content "
            "should be cached.")
    )

    class Meta:
        abstract = True


class LatestQuestionsPlugin(CMSPlugin, AdjustableCacheModelMixin,
                            QuestionsPlugin):

    def get_queryset(self):
        qs = super(LatestQuestionsPlugin, self).get_queryset()
        return qs.order_by('-id')


class TopQuestionsPlugin(CMSPlugin, AdjustableCacheModelMixin,
                         QuestionsPlugin):

    def get_queryset(self):
        qs = super(TopQuestionsPlugin, self).get_queryset()
        return qs.filter(is_top=True)


class MostReadQuestionsPlugin(CMSPlugin, AdjustableCacheModelMixin,
                              QuestionsPlugin):

    def get_queryset(self):
        qs = super(MostReadQuestionsPlugin, self).get_queryset()
        return qs.order_by('-number_of_visits')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from distutils.version import LooseVersion
from django.contrib.admin import TabularInline
from django.utils.translation import ugettext_lazy as _

from cms import __version__ as cms_version
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.utils import get_language_from_request
from adminsortable2.admin import SortableInlineAdminMixin

from . import models
from .forms import QuestionListPluginForm

CMS_GTE_330 = LooseVersion(cms_version) >= LooseVersion('3.3.0')


class FAQPlugin(CMSPluginBase):

    module = "FAQ"


class AdjustableCacheMixin(object):
    """
    For django CMS < 3.3.0 installations, we have no choice but to disable the
    cache where there is time-sensitive information. However, in later CMS
    versions, we can configure it with `get_cache_expiration()`.
    """
    if not CMS_GTE_330:
        cache = False

    def get_cache_expiration(self, request, instance, placeholder):
        return getattr(instance, 'cache_duration', 0)

    def get_fieldsets(self, request, obj=None):
        """
        Removes the cache_duration field from the displayed form if we're not
        using django CMS v3.3.0 or later.
        """
        fieldsets = super(AdjustableCacheMixin, self).get_fieldsets(
            request, obj=None)
        if CMS_GTE_330:
            return fieldsets

        field = 'cache_duration'
        for fieldset in fieldsets:
            new_fieldset = [
                item for item in fieldset[1]['fields'] if item != field]
            fieldset[1]['fields'] = tuple(new_fieldset)
        return fieldsets


class QuestionListPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/question_list.html'
    name = _('Question List')
    model = models.QuestionListPlugin
    form = QuestionListPluginForm

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class LatestQuestionsPlugin(AdjustableCacheMixin, FAQPlugin):

    render_template = 'aldryn_faq/plugins/latest_questions.html'
    name = _('Latest questions')
    model = models.LatestQuestionsPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class TopQuestionsPlugin(AdjustableCacheMixin, FAQPlugin):

    render_template = 'aldryn_faq/plugins/top_questions.html'
    name = _('Top questions')
    model = models.TopQuestionsPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class MostReadQuestionsPlugin(AdjustableCacheMixin, FAQPlugin):

    render_template = 'aldryn_faq/plugins/most_read_questions.html'
    name = _('Most read questions')
    model = models.MostReadQuestionsPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class SelectedCategoryInline(SortableInlineAdminMixin, TabularInline):
    model = models.SelectedCategory
    extra = 0

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        field = super(SelectedCategoryInline, self).formfield_for_foreignkey(
            db_field, request, **kwargs)

        if db_field.name == 'category':
            language = get_language_from_request(request)
            field.queryset = models.Category.objects.language(language)
        return field


class CategoryListPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/category_list.html'
    name = _('List of categories')
    model = models.CategoryListPlugin
    inlines = [SelectedCategoryInline]

    def render(self, context, instance, placeholder):
        context['categories'] = instance.get_categories()
        return context


plugin_pool.register_plugin(CategoryListPlugin)
plugin_pool.register_plugin(LatestQuestionsPlugin)
plugin_pool.register_plugin(QuestionListPlugin)
plugin_pool.register_plugin(TopQuestionsPlugin)
plugin_pool.register_plugin(MostReadQuestionsPlugin)



# -*- coding: utf-8 -*-
import sys
from setuptools import setup, find_packages
from aldryn_faq import __version__

py26 = (2, 7, 0) > sys.version_info >= (2, 6, 0)

REQUIREMENTS = [
    'aldryn-apphooks-config>=0.2.4',
    'aldryn-boilerplates>=0.7.4,<0.8',
    'aldryn-reversion>=1.0.4,<1.1',
    'aldryn-search',
    'aldryn-translation-tools>=0.2.1',
    'django>=1.6,<1.9.999',
    'django-admin-sortable2>=0.5.2',
    'django-cms>=3.0.12,<3.4',
    'djangocms-text-ckeditor',
    'django-parler>=1.4,<1.7',
    'django-reversion>=1.8.2,<1.11',
    'django-sortedm2m>=1.2.2',

    # THIS IS HERE TO SUPPORT EXISTING MIGRATIONS AND CAN BE REMOVED ONLY ONCE
    # WE NO LONGER SUPPORT SOUTH MIGRATIONS.
    'django-admin-sortable',
]

# use appropriate 3rd party packages versions
if py26:
    REQUIREMENTS += [
        'django-taggit<0.18.0',
    ]
else:
    REQUIREMENTS += [
        'django-taggit',
    ]

# https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Framework :: Django :: 1.6',
    'Framework :: Django :: 1.7',
    'Framework :: Django :: 1.8',
    'Framework :: Django :: 1.9',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

setup(
    name='aldryn-faq',
    version=__version__,
    description='FAQ addon for django CMS',
    author='Divio AG',
    author_email='info@divio.ch',
    url='https://github.com/aldryn/aldryn-faq',
    packages=find_packages(),
    license='LICENSE.txt',
    platforms=['OS Independent'],
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    long_description=open('README.rst').read(),
    include_package_data=True,
    zip_safe=False,
    test_suite="test_settings.run",
)
